'use strict';

var lowlight$1 = require('lowlight');
var React = require('react');
var reactivityStore = require('reactivity-store');
var shim = require('use-sync-external-store/shim');
var reactDom = require('react-dom');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var React__namespace = /*#__PURE__*/_interopNamespaceDefault(React);

const lowlight = lowlight$1.createLowlight(lowlight$1.all);
lowlight.register("vue", function hljsDefineVue(hljs) {
    return {
        subLanguage: "xml",
        contains: [
            hljs.COMMENT("<!--", "-->", {
                relevance: 10,
            }),
            {
                begin: /^(\s*)(<script>)/gm,
                end: /^(\s*)(<\/script>)/gm,
                subLanguage: "javascript",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<script\s+lang=(["'])ts\1>)/gm,
                end: /^(\s*)(<\/script>)/gm,
                subLanguage: "typescript",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(\s*)(<style(\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "css",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<style(?:\s+scoped)?\s+lang=(["'])(?:s[ca]ss)\1(?:\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "scss",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<style(?:\s+scoped)?\s+lang=(["'])stylus\1(?:\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "stylus",
                excludeBegin: true,
                excludeEnd: true,
            },
        ],
    };
});
const highlighter = lowlight;
let _autoDetectLang = true;
let _maxLineToIgnoreSyntax = 2000;
Object.defineProperty(highlighter, "maxLineToIgnoreSyntax", {
    get: () => _maxLineToIgnoreSyntax,
});
Object.defineProperty(highlighter, "setMaxLineToIgnoreSyntax", {
    value: (v) => {
        _maxLineToIgnoreSyntax = v;
    },
});
Object.defineProperty(highlighter, "autoDetectLang", {
    get: () => _autoDetectLang,
});
Object.defineProperty(highlighter, "setAutoDetectLang", {
    value: (v) => {
        _autoDetectLang = v;
    },
});

// TODO LRU Cache
const map = {};
class File {
    raw;
    lang;
    ast;
    rawFile = {};
    hasDoRaw = false;
    rawLength;
    syntaxFile = {};
    hasDoSyntax = false;
    syntaxLength;
    maxLineNumber = 0;
    constructor(raw, lang) {
        this.raw = raw;
        this.lang = lang;
        Object.defineProperty(this, "__v_skip", { value: true });
    }
    doSyntax({ autoDetectLang, registerHighlighter }) {
        if (!this.raw || this.hasDoSyntax)
            return;
        let hasRegisteredLang = true;
        const _highlighter = registerHighlighter || highlighter;
        if (this.syntaxLength) {
            console.error("current file already doSyntax before!");
            return;
        }
        if (!_highlighter.registered(this.lang)) {
            hasRegisteredLang = false;
            if (!autoDetectLang) {
                console.warn(`not support current lang: ${this.lang} yet`);
                return;
            }
        }
        if (this.rawLength > _highlighter.maxLineToIgnoreSyntax) {
            console.warn(`ignore syntax for current file, because the rawLength is too long: ${this.rawLength}`);
            return;
        }
        if (hasRegisteredLang) {
            this.ast = _highlighter.highlight(this.lang, this.raw);
        }
        else {
            this.ast = _highlighter.highlightAuto(this.raw);
        }
        this.#doAST();
        {
            this.#doCheck();
        }
        this.hasDoSyntax = true;
    }
    doRaw() {
        if (!this.raw || this.hasDoRaw)
            return;
        const rawString = this.raw;
        const rawArray = rawString.split("\n");
        this.rawLength = rawArray.length;
        this.maxLineNumber = rawArray.length;
        this.rawFile = rawArray.reduce((p, item, index) => ({
            ...p,
            [index + 1]: index < rawArray.length - 1 ? item + "\n" : item,
        }), {});
        this.hasDoRaw = true;
    }
    #doAST() {
        const ast = this.ast;
        let lineNumber = 1;
        const syntaxObj = this.syntaxFile;
        const loopAST = (nodes, wrapper) => {
            nodes.forEach((node) => {
                if (node.type === "text") {
                    if (node.value.indexOf("\n") === -1) {
                        const valueLength = node.value.length;
                        if (!syntaxObj[lineNumber]) {
                            node.startIndex = 0;
                            node.endIndex = valueLength - 1;
                            const item = {
                                value: node.value,
                                lineNumber,
                                valueLength,
                                nodeList: [{ node, wrapper }],
                            };
                            syntaxObj[lineNumber] = item;
                        }
                        else {
                            node.startIndex = syntaxObj[lineNumber].valueLength;
                            node.endIndex = node.startIndex + valueLength - 1;
                            syntaxObj[lineNumber].value += node.value;
                            syntaxObj[lineNumber].valueLength += valueLength;
                            syntaxObj[lineNumber].nodeList.push({ node, wrapper });
                        }
                        node.lineNumber = lineNumber;
                        return;
                    }
                    const lines = node.value.split("\n");
                    node.children = node.children || [];
                    for (let i = 0; i < lines.length; i++) {
                        const _value = i === lines.length - 1 ? lines[i] : lines[i] + "\n";
                        const _lineNumber = i === 0 ? lineNumber : ++lineNumber;
                        const _valueLength = _value.length;
                        const _node = {
                            type: "text",
                            value: _value,
                            startIndex: Infinity,
                            endIndex: Infinity,
                            lineNumber: _lineNumber,
                        };
                        if (!syntaxObj[_lineNumber]) {
                            _node.startIndex = 0;
                            _node.endIndex = _valueLength - 1;
                            const item = {
                                value: _value,
                                lineNumber: _lineNumber,
                                valueLength: _valueLength,
                                nodeList: [{ node: _node, wrapper }],
                            };
                            syntaxObj[_lineNumber] = item;
                        }
                        else {
                            _node.startIndex = syntaxObj[_lineNumber].valueLength;
                            _node.endIndex = _node.startIndex + _valueLength - 1;
                            syntaxObj[_lineNumber].value += _value;
                            syntaxObj[_lineNumber].valueLength += _valueLength;
                            syntaxObj[_lineNumber].nodeList.push({ node: _node, wrapper });
                        }
                        node.children.push(_node);
                    }
                    node.lineNumber = lineNumber;
                    return;
                }
                if (node.children) {
                    loopAST(node.children, node);
                    node.lineNumber = lineNumber;
                }
            });
        };
        loopAST(ast.children);
        this.syntaxLength = lineNumber;
    }
    #doCheck() {
        if (this.rawLength && this.syntaxLength) {
            if (this.rawLength !== this.syntaxLength) {
                console.warn("the rawLength not match for the syntaxLength");
            }
            Object.values(this.syntaxFile).forEach(({ value, lineNumber }) => {
                if (value !== this.rawFile[lineNumber]) {
                    console.log("some line not match:" + value + " __ " + this.rawFile[lineNumber] + " __ at: " + lineNumber + " lineNumber");
                }
            });
        }
    }
}
const getFile = (raw, lang) => {
    const key = raw + "--" + "0.0.2" + "--" + lang;
    if (map[key])
        return map[key];
    const file = new File(raw, lang);
    map[key] = file;
    return file;
};

/** indicate what a line in the diff represents */
var DiffLineType;
(function (DiffLineType) {
    DiffLineType[DiffLineType["Context"] = 0] = "Context";
    DiffLineType[DiffLineType["Add"] = 1] = "Add";
    DiffLineType[DiffLineType["Delete"] = 2] = "Delete";
    DiffLineType[DiffLineType["Hunk"] = 3] = "Hunk";
})(DiffLineType || (DiffLineType = {}));
/** track details related to each line in the diff */
class DiffLine {
    text;
    type;
    originalLineNumber;
    oldLineNumber;
    newLineNumber;
    noTrailingNewLine;
    needRematch;
    range;
    constructor(text, type, 
    // Line number in the original diff patch (before expanding it), or null if
    // it was added as part of a diff expansion action.
    originalLineNumber, oldLineNumber, newLineNumber, noTrailingNewLine = false, needRematch = false, range) {
        this.text = text;
        this.type = type;
        this.originalLineNumber = originalLineNumber;
        this.oldLineNumber = oldLineNumber;
        this.newLineNumber = newLineNumber;
        this.noTrailingNewLine = noTrailingNewLine;
        this.needRematch = needRematch;
        this.range = range;
    }
    withNoTrailingNewLine(noTrailingNewLine) {
        return new DiffLine(this.text, this.type, this.originalLineNumber, this.oldLineNumber, this.newLineNumber, noTrailingNewLine);
    }
    isIncludeableLine() {
        return this.type === DiffLineType.Add || this.type === DiffLineType.Delete;
    }
    /** The content of the line, i.e., without the line type marker. */
    get content() {
        return this.text.substring(1);
    }
    equals(other) {
        return (this.text === other.text &&
            this.type === other.type &&
            this.originalLineNumber === other.originalLineNumber &&
            this.oldLineNumber === other.oldLineNumber &&
            this.newLineNumber === other.newLineNumber &&
            this.noTrailingNewLine === other.noTrailingNewLine);
    }
}
const checkDiffLineIncludeChange = (diffLine) => {
    if (!diffLine)
        return false;
    return diffLine.type === DiffLineType.Add || diffLine.type === DiffLineType.Delete;
};

var DiffHunkExpansionType;
(function (DiffHunkExpansionType) {
    /** The hunk header cannot be expanded at all. */
    DiffHunkExpansionType["None"] = "None";
    /**
     * The hunk header can be expanded up exclusively. Only the first hunk can be
     * expanded up exclusively.
     */
    DiffHunkExpansionType["Up"] = "Up";
    /**
     * The hunk header can be expanded down exclusively. Only the last hunk (if
     * it's the dummy hunk with only one line) can be expanded down exclusively.
     */
    DiffHunkExpansionType["Down"] = "Down";
    /** The hunk header can be expanded both up and down. */
    DiffHunkExpansionType["Both"] = "Both";
    /**
     * The hunk header represents a short gap that, when expanded, will
     * result in merging this hunk and the hunk above.
     */
    DiffHunkExpansionType["Short"] = "Short";
})(DiffHunkExpansionType || (DiffHunkExpansionType = {}));
/** each diff is made up of a number of hunks */
class DiffHunk {
    header;
    lines;
    unifiedDiffStart;
    unifiedDiffEnd;
    expansionType;
    /**
     * @param header The details from the diff hunk header about the line start and patch length.
     * @param lines The contents - context and changes - of the diff section.
     * @param unifiedDiffStart The diff hunk's start position in the overall file diff.
     * @param unifiedDiffEnd The diff hunk's end position in the overall file diff.
     */
    constructor(header, lines, unifiedDiffStart, unifiedDiffEnd, expansionType) {
        this.header = header;
        this.lines = lines;
        this.unifiedDiffStart = unifiedDiffStart;
        this.unifiedDiffEnd = unifiedDiffEnd;
        this.expansionType = expansionType;
    }
    equals(other) {
        if (this === other) {
            return true;
        }
        return (this.header.equals(other.header) &&
            this.unifiedDiffStart === other.unifiedDiffStart &&
            this.unifiedDiffEnd === other.unifiedDiffEnd &&
            this.expansionType === other.expansionType &&
            this.lines.length === other.lines.length &&
            this.lines.every((xLine, ix) => xLine.equals(other.lines[ix])));
    }
}
/** details about the start and end of a diff hunk */
class DiffHunkHeader {
    oldStartLine;
    oldLineCount;
    newStartLine;
    newLineCount;
    /**
     * @param oldStartLine The line in the old (or original) file where this diff hunk starts.
     * @param oldLineCount The number of lines in the old (or original) file that this diff hunk covers
     * @param newStartLine The line in the new file where this diff hunk starts.
     * @param newLineCount The number of lines in the new file that this diff hunk covers.
     */
    constructor(oldStartLine, oldLineCount, newStartLine, newLineCount) {
        this.oldStartLine = oldStartLine;
        this.oldLineCount = oldLineCount;
        this.newStartLine = newStartLine;
        this.newLineCount = newLineCount;
    }
    toDiffLineRepresentation() {
        return `@@ -${this.oldStartLine},${this.oldLineCount} +${this.newStartLine},${this.newLineCount} @@`;
    }
    equals(other) {
        return (this.oldStartLine === other.oldStartLine &&
            this.oldLineCount === other.oldLineCount &&
            this.newStartLine === other.newStartLine &&
            this.oldStartLine === other.oldStartLine);
    }
}

/** How many new lines will be added to a diff hunk by default. */
const DefaultDiffExpansionStep = 20;
function assertNever(_, message) {
    throw new Error(message);
}
/** Utility function for getting the digit count of the largest line number in an array of diff hunks */
function getLargestLineNumber(hunks) {
    if (hunks.length === 0) {
        return 0;
    }
    for (let i = hunks.length - 1; i >= 0; i--) {
        const hunk = hunks[i];
        for (let j = hunk.lines.length - 1; j >= 0; j--) {
            const line = hunk.lines[j];
            if (line.type === DiffLineType.Hunk) {
                continue;
            }
            const newLineNumber = line.newLineNumber ?? 0;
            const oldLineNumber = line.oldLineNumber ?? 0;
            return newLineNumber > oldLineNumber ? newLineNumber : oldLineNumber;
        }
    }
    return 0;
}
/**
 * Calculates whether or not a hunk header can be expanded up, down, both, or if
 * the space represented by the hunk header is short and expansion there would
 * mean merging with the hunk above.
 *
 * @param hunkIndex     Index of the hunk to evaluate within the whole diff.
 * @param hunkHeader    Header of the hunk to evaluate.
 * @param previousHunk  Hunk previous to the one to evaluate. Null if the
 *                      evaluated hunk is the first one.
 */
function getHunkHeaderExpansionType(hunkIndex, hunkHeader, previousHunk) {
    const distanceToPrevious = previousHunk === null
        ? Infinity
        : hunkHeader.oldStartLine - previousHunk.header.oldStartLine - previousHunk.header.oldLineCount;
    // In order to simplify the whole logic around expansion, only the hunk at the
    // top can be expanded up exclusively, and only the hunk at the bottom (the
    // dummy one, see getTextDiffWithBottomDummyHunk) can be expanded down
    // exclusively.
    // The rest of the hunks can be expanded both ways, except those which are too
    // short and therefore the direction of expansion doesn't matter.
    if (hunkIndex === 0) {
        // The top hunk can only be expanded if there is content above it
        if (hunkHeader.oldStartLine > 1 && hunkHeader.newStartLine > 1) {
            return DiffHunkExpansionType.Up;
        }
        else {
            return DiffHunkExpansionType.None;
        }
    }
    else if (distanceToPrevious <= DefaultDiffExpansionStep) {
        return DiffHunkExpansionType.Short;
    }
    else {
        return DiffHunkExpansionType.Both;
    }
}
const numIterator = (num, cb) => {
    const re = [];
    for (let i = 0; i < num; i++) {
        re.push(cb(i));
    }
    return re;
};

const maxLength = 1000;
/** Get the maximum position in the range. */
function rangeMax(range) {
    return range.location + range.length;
}
/** Get the length of the common substring between the two strings. */
function commonLength(stringA, rangeA, stringB, rangeB, reverse) {
    const max = Math.min(rangeA.length, rangeB.length);
    const startA = reverse ? rangeMax(rangeA) - 1 : rangeA.location;
    const startB = reverse ? rangeMax(rangeB) - 1 : rangeB.location;
    const stride = reverse ? -1 : 1;
    let length = 0;
    while (Math.abs(length) < max) {
        if (stringA[startA + length] !== stringB[startB + length]) {
            break;
        }
        length += stride;
    }
    return Math.abs(length);
}
function isInValidString(s) {
    return s.trim().length === 0 || s.length >= maxLength;
}
/** Get the changed ranges in the strings, relative to each other. */
function relativeChanges(stringA, stringB) {
    let bRange = { location: 0, length: stringB.length };
    let aRange = { location: 0, length: stringA.length };
    const _stringA = stringA.trimEnd();
    const _stringB = stringB.trimEnd();
    if (_stringA === _stringB) {
        return {
            stringARange: { location: _stringA.length, length: stringA.length - _stringA.length },
            stringBRange: { location: _stringB.length, length: stringB.length - _stringB.length },
        };
    }
    if (isInValidString(stringA) || isInValidString(stringB)) {
        aRange.length = 0;
        bRange.length = 0;
        return { stringARange: aRange, stringBRange: bRange };
    }
    const prefixLength = commonLength(stringB, bRange, stringA, aRange, false);
    bRange = {
        location: bRange.location + prefixLength,
        length: bRange.length - prefixLength,
    };
    aRange = {
        location: aRange.location + prefixLength,
        length: aRange.length - prefixLength,
    };
    const suffixLength = commonLength(stringB, bRange, stringA, aRange, true);
    bRange.length -= suffixLength;
    aRange.length -= suffixLength;
    return { stringARange: aRange, stringBRange: bRange };
}

/* eslint-disable max-lines */
// NODE: ALL of the Diff parse logic from desktop, SEE https://github.com/desktop/desktop
// https://en.wikipedia.org/wiki/Diff_utility
//
// @@ -l,s +l,s @@ optional section heading
//
// The hunk range information contains two hunk ranges. The range for the hunk of the original
// file is preceded by a minus symbol, and the range for the new file is preceded by a plus
// symbol. Each hunk range is of the format l,s where l is the starting line number and s is
// the number of lines the change hunk applies to for each respective file.
//
// In many versions of GNU diff, each range can omit the comma and trailing value s,
// in which case s defaults to 1
const diffHeaderRe = /^@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@/;
/**
 * Regular expression matching invisible bidirectional Unicode characters that
 * may be interpreted or compiled differently than what it appears. More info:
 * https://github.co/hiddenchars
 */
const HiddenBidiCharsRegex = /[\u202A-\u202E]|[\u2066-\u2069]/;
const DiffPrefixAdd = "+";
const DiffPrefixDelete = "-";
const DiffPrefixContext = " ";
const DiffPrefixNoNewline = "\\";
const DiffLinePrefixChars = new Set([
    DiffPrefixAdd,
    DiffPrefixDelete,
    DiffPrefixContext,
    DiffPrefixNoNewline,
]);
/**
 * A parser for the GNU unified diff format
 *
 * See https://www.gnu.org/software/diffutils/manual/html_node/Detailed-Unified.html
 */
class DiffParser {
    /**
     * Line start pointer.
     *
     * The offset into the text property where the current line starts (ie either zero
     * or one character ahead of the last newline character).
     */
    ls;
    /**
     * Line end pointer.
     *
     * The offset into the text property where the current line ends (ie it points to
     * the newline character) or -1 if the line boundary hasn't been determined yet
     */
    le;
    /**
     * The text buffer containing the raw, unified diff output to be parsed
     */
    text;
    constructor() {
        Object.defineProperty(this, "__v_skip", { value: true });
        this.reset();
    }
    /**
     * Resets the internal parser state so that it can be reused.
     *
     * This is done automatically at the end of each parse run.
     */
    reset() {
        this.ls = 0;
        this.le = -1;
        this.text = "";
    }
    /**
     * Aligns the internal character pointers at the boundaries of
     * the next line.
     *
     * Returns true if successful or false if the end of the diff
     * has been reached.
     */
    nextLine() {
        this.ls = this.le + 1;
        // We've reached the end of the diff
        if (this.ls >= this.text.length) {
            return false;
        }
        this.le = this.text.indexOf("\n", this.ls);
        // If we can't find the next newline character we'll put our
        // end pointer at the end of the diff string
        if (this.le === -1) {
            this.le = this.text.length;
        }
        // We've succeeded if there's anything to read in between the
        // start and the end
        return this.ls !== this.le;
    }
    /**
     * Advances to the next line and returns it as a substring
     * of the raw diff text. Returns null if end of diff was
     * reached.
     */
    readLine(header) {
        if (header) {
            return this.nextLine() ? this.text.substring(this.ls, this.le) : null;
        }
        else {
            return this.nextLine() ? this.text.substring(this.ls + 1, this.le + 1) : null;
        }
    }
    /** Tests if the current line starts with the given search text */
    lineStartsWith(searchString) {
        return this.text.startsWith(searchString, this.ls);
    }
    /** Tests if the current line ends with the given search text */
    lineEndsWith(searchString) {
        return this.text.endsWith(searchString, this.le);
    }
    /**
     * Returns the starting character of the next line without
     * advancing the internal state. Returns null if advancing
     * would mean reaching the end of the diff.
     */
    peek() {
        const p = this.le + 1;
        return p < this.text.length ? this.text[p] : null;
    }
    /**
     * Parse the diff header, meaning everything from the
     * start of the diff output to the end of the line beginning
     * with +++
     *
     * Example diff header:
     *
     *   diff --git a/app/src/lib/diff-parser.ts b/app/src/lib/diff-parser.ts
     *   index e1d4871..3bd3ee0 100644
     *   --- a/app/src/lib/diff-parser.ts
     *   +++ b/app/src/lib/diff-parser.ts
     *
     * Returns an object with information extracted from the diff
     * header (currently whether it's a binary patch) or null if
     * the end of the diff was reached before the +++ line could be
     * found (which is a valid state).
     */
    parseDiffHeader() {
        // TODO: There's information in here that we might want to
        // capture, such as mode changes
        while (this.nextLine()) {
            if (this.lineStartsWith("Binary files ") && this.lineEndsWith("differ")) {
                return { isBinary: true };
            }
            if (this.lineStartsWith("+++")) {
                return { isBinary: false };
            }
        }
        // It's not an error to not find the +++ line, see the
        // 'parses diff of empty file' test in diff-parser-tests.ts
        return null;
    }
    /**
     * Attempts to convert a RegExp capture group into a number.
     * If the group doesn't exist or wasn't captured the function
     * will return the value of the defaultValue parameter or throw
     * an error if no default value was provided. If the captured
     * string can't be converted to a number an error will be thrown.
     */
    numberFromGroup(m, group, defaultValue = null) {
        const str = m[group];
        if (!str) {
            if (!defaultValue) {
                throw new Error(`Group ${group} missing from regexp match and no defaultValue was provided`);
            }
            return defaultValue;
        }
        const num = parseInt(str, 10);
        if (isNaN(num)) {
            throw new Error(`Could not parse capture group ${group} into number: ${str}`);
        }
        return num;
    }
    /**
     * Parses a hunk header or throws an error if the given line isn't
     * a well-formed hunk header.
     *
     * We currently only extract the line number information and
     * ignore any hunk headings.
     *
     * Example hunk header (text within ``):
     *
     * `@@ -84,10 +82,8 @@ export function parseRawDiff(lines: ReadonlyArray<string>): Diff {`
     *
     * Where everything after the last @@ is what's known as the hunk, or section, heading
     */
    parseHunkHeader(line) {
        const m = diffHeaderRe.exec(line);
        if (!m) {
            throw new Error(`Invalid hunk header format`);
        }
        // If endLines are missing default to 1, see diffHeaderRe docs
        const oldStartLine = this.numberFromGroup(m, 1);
        const oldLineCount = this.numberFromGroup(m, 2, 1);
        const newStartLine = this.numberFromGroup(m, 3);
        const newLineCount = this.numberFromGroup(m, 4, 1);
        return new DiffHunkHeader(oldStartLine, oldLineCount, newStartLine, newLineCount);
    }
    /**
     * Convenience function which lets us leverage the type system to
     * prove exhaustive checks in parseHunk.
     *
     * Takes an arbitrary string and checks to see if the first character
     * of that string is one of the allowed prefix characters for diff
     * lines (ie lines in between hunk headers).
     */
    parseLinePrefix(c) {
        // Since we know that DiffLinePrefixChars and the DiffLinePrefix type
        // include the same characters we can tell the type system that we
        // now know that c[0] is one of the characters in the DifflinePrefix set
        if (c && c.length && DiffLinePrefixChars.has(c[0])) {
            return c[0];
        }
        return null;
    }
    /**
     * Parses a hunk, including its header or throws an error if the diff doesn't
     * contain a well-formed diff hunk at the current position.
     *
     * Expects that the position has been advanced to the beginning of a presumed
     * diff hunk header.
     *
     * @param linesConsumed The number of unified diff lines consumed up until
     *                      this point by the diff parser. Used to give the
     *                      position and length (in lines) of the parsed hunk
     *                      relative to the overall parsed diff. These numbers
     *                      have no real meaning in the context of a diff and
     *                      are only used to aid the app in line-selections.
     */
    parseHunk(linesConsumed, hunkIndex, previousHunk) {
        const headerLine = this.readLine(true);
        if (!headerLine) {
            throw new Error("Expected hunk header but reached end of diff");
        }
        const header = this.parseHunkHeader(headerLine);
        const lines = new Array();
        lines.push(new DiffLine(headerLine, DiffLineType.Hunk, 1, null, null));
        let c;
        let rollingDiffBeforeCounter = header.oldStartLine;
        let rollingDiffAfterCounter = header.newStartLine;
        let diffLineNumber = linesConsumed;
        while ((c = this.parseLinePrefix(this.peek()))) {
            const line = this.readLine(false);
            diffLineNumber++;
            if (!line) {
                throw new Error("Expected unified diff line but reached end of diff");
            }
            // A marker indicating that the last line in the original or the new file
            // is missing a trailing newline. In other words, the presence of this marker
            // means that the new and/or original file lacks a trailing newline.
            //
            // When we find it we have to look up the previous line and set the
            // noTrailingNewLine flag
            if (c === DiffPrefixNoNewline) {
                // See https://github.com/git/git/blob/21f862b498925194f8f1ebe8203b7a7df756555b/apply.c#L1725-L1732
                if (line.length < 12) {
                    throw new Error(`Expected "no newline at end of file" marker to be at least 12 bytes long`);
                }
                const previousLineIndex = lines.length - 1;
                const previousLine = lines[previousLineIndex];
                lines[previousLineIndex] = previousLine.withNoTrailingNewLine(true);
                continue;
            }
            let diffLine;
            if (c === DiffPrefixAdd) {
                diffLine = new DiffLine(line, DiffLineType.Add, diffLineNumber, null, rollingDiffAfterCounter++);
            }
            else if (c === DiffPrefixDelete) {
                diffLine = new DiffLine(line, DiffLineType.Delete, diffLineNumber, rollingDiffBeforeCounter++, null);
            }
            else if (c === DiffPrefixContext) {
                diffLine = new DiffLine(line, DiffLineType.Context, diffLineNumber, rollingDiffBeforeCounter++, rollingDiffAfterCounter++);
            }
            else {
                return assertNever(c, `Unknown DiffLinePrefix: ${c}`);
            }
            lines.push(diffLine);
        }
        if (lines.length === 1) {
            throw new Error("Malformed diff, empty hunk");
        }
        return new DiffHunk(header, lines, linesConsumed, linesConsumed + lines.length - 1, getHunkHeaderExpansionType(hunkIndex, header, previousHunk));
    }
    /**
     * Parse a well-formed unified diff into hunks and lines.
     *
     * @param text A unified diff produced by git diff, git log --patch
     *             or any other git plumbing command that produces unified
     *             diffs.
     */
    parse(text) {
        this.text = text;
        try {
            const headerInfo = this.parseDiffHeader();
            const headerEnd = this.le;
            const header = this.text.substring(0, headerEnd);
            // empty diff
            if (!headerInfo) {
                return {
                    header,
                    contents: "",
                    hunks: [],
                    isBinary: false,
                    maxLineNumber: 0,
                    hasHiddenBidiChars: false,
                };
            }
            if (headerInfo.isBinary) {
                return {
                    header,
                    contents: "",
                    hunks: [],
                    isBinary: true,
                    maxLineNumber: 0,
                    hasHiddenBidiChars: false,
                };
            }
            const hunks = new Array();
            let linesConsumed = 0;
            let previousHunk = null;
            do {
                const hunk = this.parseHunk(linesConsumed, hunks.length, previousHunk);
                hunks.push(hunk);
                previousHunk = hunk;
                linesConsumed += hunk.lines.length;
            } while (this.peek());
            const contents = this.text
                .substring(headerEnd + 1, this.le)
                // Note that this simply returns a reference to the
                // substring if no match is found, it does not create
                // a new string instance.
                .replace(/\n\\ No newline at end of file/g, "");
            return {
                header,
                contents,
                hunks,
                isBinary: headerInfo.isBinary,
                maxLineNumber: getLargestLineNumber(hunks),
                hasHiddenBidiChars: HiddenBidiCharsRegex.test(text),
            };
        }
        finally {
            this.reset();
        }
    }
}
const parseInstance = new DiffParser();

/* eslint-disable @typescript-eslint/ban-ts-comment */
/* eslint-disable max-lines */
const composeLen = 40;
const idSet = new Set();
const getLang = (fileName) => {
    const dotIndex = fileName.lastIndexOf(".");
    const extension = fileName.slice(dotIndex + 1);
    return extension;
};
class DiffFile {
    _oldFileName;
    _newFileName;
    _diffList;
    #oldFileResult;
    #newFileResult;
    #diffListResults;
    #diffLines;
    #oldFileDiffLines;
    #newFileDiffLines;
    #oldFileLines;
    #newFileLines;
    #oldFileSyntaxLines;
    #newFileSyntaxLines;
    #splitLeftLines = [];
    #splitRightLines = [];
    #splitHunksLines;
    #splitLastStartIndex;
    #unifiedLines = [];
    #unifiedHunksLines;
    #unifiedLastStartIndex;
    #listeners = [];
    #hasInitRaw = false;
    #hasInitSyntax = false;
    #hasBuildSplit = false;
    #hasBuildUnified = false;
    #updateCount = 0;
    #composeByDiff = false;
    _oldFileContent = "";
    _oldFileLang = "";
    _newFileContent = "";
    _newFileLang = "";
    diffLineLength = 0;
    splitLineLength = 0;
    unifiedLineLength = 0;
    #id = "";
    static createInstance(data, bundle) {
        const instance = new DiffFile(data?.oldFile?.fileName || "", data?.oldFile?.content || "", data?.newFile?.fileName || "", data?.newFile?.content || "", data?.hunks || [], data?.oldFile?.fileLang || "", data?.newFile?.fileLang || "");
        if (bundle) {
            instance.mergeBundle(bundle);
        }
        return instance;
    }
    constructor(_oldFileName, _oldFileContent, _newFileName, _newFileContent, _diffList, _oldFileLang, _newFileLang) {
        this._oldFileName = _oldFileName;
        this._newFileName = _newFileName;
        this._diffList = _diffList;
        Object.defineProperty(this, "__v_skip", { value: true });
        let oldContent = _oldFileContent;
        let newContent = _newFileContent;
        const diffList = Array.from(new Set(_diffList));
        Object.defineProperties(this, {
            _oldFileName: { get: () => _oldFileName },
            _newFileName: { get: () => _newFileName },
            _oldFileLang: { get: () => getLang(_oldFileLang || _oldFileName || _newFileLang || _newFileName) || "txt" },
            _newFileLang: { get: () => getLang(_newFileLang || _newFileName || _oldFileLang || _oldFileName) || "txt" },
            _oldFileContent: {
                get: () => oldContent,
                set: (v) => (oldContent = v),
            },
            _newFileContent: {
                get: () => newContent,
                set: (v) => (newContent = v),
            },
            _diffList: { get: () => diffList },
        });
        this.initId();
    }
    #doDiff() {
        if (!this._diffList)
            return;
        this.#diffListResults = this._diffList.map((s) => parseInstance.parse(s));
    }
    #doFile() {
        if (!this._oldFileContent && !this._newFileContent)
            return;
        if (this._oldFileContent) {
            this.#oldFileResult = getFile(this._oldFileContent, this._oldFileLang);
        }
        if (this._newFileContent) {
            this.#newFileResult = getFile(this._newFileContent, this._newFileLang);
        }
    }
    #composeRaw() {
        this.#oldFileResult?.doRaw();
        this.#oldFileLines = this.#oldFileResult?.rawFile;
        this.#newFileResult?.doRaw();
        this.#newFileLines = this.#newFileResult?.rawFile;
    }
    #composeFile() {
        if (this._oldFileContent && this._newFileContent)
            return;
        // all of the file content not exist, try to use diff result to compose
        if (!this._oldFileContent && !this._newFileContent) {
            let newLineNumber = 1;
            let oldLineNumber = 1;
            let oldFileContent = "";
            let newFileContent = "";
            while (oldLineNumber <= this.diffLineLength) {
                const diffLine = this.#getOldDiffLine(oldLineNumber++);
                if (diffLine) {
                    oldFileContent += diffLine.text;
                }
                else {
                    // empty line for placeholder
                    oldFileContent += "\n";
                }
            }
            while (newLineNumber <= this.diffLineLength) {
                const diffLine = this.#getNewDiffLine(newLineNumber++);
                if (diffLine) {
                    newFileContent += diffLine.text;
                }
                else {
                    // empty line for placeholder
                    newFileContent += "\n";
                }
            }
            if (oldFileContent === newFileContent)
                return;
            this._oldFileContent = oldFileContent;
            this._newFileContent = newFileContent;
            this.#oldFileResult = getFile(this._oldFileContent, this._oldFileLang);
            this.#newFileResult = getFile(this._newFileContent, this._newFileLang);
            // all of the file just compose by diff, so we can not do the expand action
            this.#composeByDiff = true;
        }
        else if (this.#oldFileResult) {
            let newLineNumber = 1;
            let oldLineNumber = 1;
            let newFileContent = "";
            while (oldLineNumber <= this.#oldFileResult.maxLineNumber) {
                const newDiffLine = this.#getNewDiffLine(newLineNumber++);
                if (newDiffLine) {
                    newFileContent += newDiffLine.text;
                    oldLineNumber = newDiffLine.oldLineNumber ? newDiffLine.oldLineNumber + 1 : oldLineNumber;
                }
                else {
                    newFileContent += this.#getOldRawLine(oldLineNumber++);
                }
            }
            if (newFileContent === this._oldFileContent)
                return;
            this._newFileContent = newFileContent;
            this.#newFileResult = getFile(this._newFileContent, this._newFileLang);
        }
        else if (this.#newFileResult) {
            let oldLineNumber = 1;
            let newLineNumber = 1;
            let oldFileContent = "";
            while (newLineNumber <= this.#newFileResult.maxLineNumber) {
                const oldDiffLine = this.#getOldDiffLine(oldLineNumber++);
                if (oldDiffLine) {
                    oldFileContent += oldDiffLine.text;
                    newLineNumber = oldDiffLine.newLineNumber ? oldDiffLine.newLineNumber + 1 : newLineNumber;
                }
                else {
                    oldFileContent += this.#getNewRawLine(newLineNumber++);
                }
            }
            if (oldFileContent === this._newFileContent)
                return;
            this._oldFileContent = oldFileContent;
            this.#oldFileResult = getFile(this._oldFileContent, this._oldFileLang);
        }
        this.#composeRaw();
    }
    #composeDiff() {
        if (!this.#diffListResults?.length)
            return;
        this.#diffListResults.forEach((item) => {
            const hunks = item.hunks;
            hunks.forEach((hunk) => {
                let additions = [];
                let deletions = [];
                hunk.lines.forEach((line) => {
                    if (line.type === DiffLineType.Add) {
                        additions.push(line);
                    }
                    else if (line.type === DiffLineType.Delete) {
                        deletions.push(line);
                    }
                    else {
                        if (additions.length === deletions.length) {
                            const len = additions.length;
                            for (let i = 0; i < len; i++) {
                                const addition = additions[i];
                                const deletion = deletions[i];
                                const { stringARange, stringBRange } = relativeChanges(addition.text, deletion.text);
                                addition.needRematch = true;
                                addition.range = stringARange;
                                deletion.needRematch = true;
                                deletion.range = stringBRange;
                            }
                        }
                        additions = [];
                        deletions = [];
                    }
                });
            });
        });
        this.#diffLines = this.#diffListResults
            .reduce((p, c) => p.concat(...c.hunks.reduce((_p, _c) => _p.concat(..._c.lines), [])), [])
            .map((i, index) => {
            const typedI = i;
            typedI.index = index;
            if (typedI.type === DiffLineType.Hunk) {
                const numInfo = typedI.text.split("@@")?.[1].split(" ").filter(Boolean);
                const oldNumInfo = numInfo?.[0] || "";
                const newNumInfo = numInfo?.[1] || "";
                const [oldNumStartIndex, oldNumLength] = oldNumInfo.split(",");
                const [newNumStartIndex, newNumLength] = newNumInfo.split(",");
                const typedTypeI = typedI;
                typedTypeI.hunkInfo = {
                    oldStartIndex: -Number(oldNumStartIndex),
                    oldLength: Number(oldNumLength),
                    newStartIndex: +Number(newNumStartIndex),
                    newLength: Number(newNumLength),
                };
            }
            return typedI;
        });
        this.#oldFileDiffLines = this.#diffLines.reduce((p, c) => {
            if (c.oldLineNumber) {
                this.diffLineLength = Math.max(this.diffLineLength, c.oldLineNumber);
                return { ...p, [c.oldLineNumber]: c };
            }
            else {
                return p;
            }
        }, {});
        this.#newFileDiffLines = this.#diffLines.reduce((p, c) => {
            if (c.newLineNumber) {
                this.diffLineLength = Math.max(this.diffLineLength, c.newLineNumber);
                return { ...p, [c.newLineNumber]: c };
            }
            else {
                return p;
            }
        }, {});
    }
    #composeSyntax({ autoDetectLang, registerHighlighter, }) {
        this.#oldFileResult?.doSyntax({ autoDetectLang, registerHighlighter });
        this.#oldFileSyntaxLines = this.#oldFileResult?.syntaxFile;
        this.#newFileResult?.doSyntax({ autoDetectLang, registerHighlighter });
        this.#newFileSyntaxLines = this.#newFileResult?.syntaxFile;
    }
    #getOldDiffLine(lineNumber) {
        if (!lineNumber)
            return;
        return this.#oldFileDiffLines?.[lineNumber];
    }
    #getNewDiffLine(lineNumber) {
        if (!lineNumber)
            return;
        return this.#newFileDiffLines?.[lineNumber];
    }
    #getOldRawLine(lineNumber) {
        return this.#oldFileLines?.[lineNumber];
    }
    #getNewRawLine(lineNumber) {
        return this.#newFileLines?.[lineNumber];
    }
    initId() {
        let id = "--" + Math.random().toString().slice(2);
        while (idSet.has(id)) {
            id = "--" + Math.random().toString().slice(2);
        }
        idSet.add(id);
        this.#id = id;
    }
    getId() {
        return this.#id;
    }
    clearId() {
        idSet.delete(this.#id);
    }
    initRaw() {
        if (this.#hasInitRaw)
            return;
        this.#doDiff();
        this.#composeDiff();
        this.#doFile();
        this.#composeRaw();
        this.#composeFile();
        this.#hasInitRaw = true;
    }
    initSyntax({ autoDetectLang, registerHighlighter, } = {}) {
        if (this.#hasInitSyntax)
            return;
        this.#composeSyntax({ registerHighlighter, autoDetectLang });
        this.#hasInitSyntax = true;
    }
    init() {
        this.initRaw();
        this.initSyntax();
    }
    buildSplitDiffLines() {
        if (this.#hasBuildSplit)
            return;
        let oldFileLineNumber = 1;
        let newFileLineNumber = 1;
        let prevIsHidden = false;
        let hideStart = Infinity;
        const maxOldFileLineNumber = this.#oldFileResult?.maxLineNumber || 0;
        const maxNewFileLineNumber = this.#newFileResult?.maxLineNumber || 0;
        while (oldFileLineNumber <= maxOldFileLineNumber || newFileLineNumber <= maxNewFileLineNumber) {
            const oldRawLine = this.#getOldRawLine(oldFileLineNumber);
            const oldDiffLine = this.#getOldDiffLine(oldFileLineNumber);
            const newRawLine = this.#getNewRawLine(newFileLineNumber);
            const newDiffLine = this.#getNewDiffLine(newFileLineNumber);
            const oldLineHasChange = oldDiffLine?.isIncludeableLine();
            const newLineHasChange = newDiffLine?.isIncludeableLine();
            const len = this.#splitRightLines.length;
            const isHidden = !oldDiffLine && !newDiffLine;
            if (!oldDiffLine && !newRawLine && !oldDiffLine && !newDiffLine)
                break;
            if ((oldLineHasChange && newLineHasChange) || (!oldLineHasChange && !newLineHasChange)) {
                this.#splitLeftLines.push({
                    lineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                });
                this.#splitRightLines.push({
                    lineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                });
            }
            else if (oldLineHasChange) {
                this.#splitLeftLines.push({
                    lineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                });
                this.#splitRightLines.push({});
            }
            else if (newLineHasChange) {
                this.#splitLeftLines.push({});
                this.#splitRightLines.push({
                    lineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                });
            }
            if (!prevIsHidden && isHidden) {
                hideStart = len;
            }
            prevIsHidden = isHidden;
            if (oldDiffLine && newDiffLine && !oldLineHasChange && !newLineHasChange) {
                const current = newDiffLine;
                const previous = newDiffLine.index ? this.#diffLines?.[current.index - 1] : undefined;
                if (previous && previous.type === DiffLineType.Hunk) {
                    const typedPrevious = previous;
                    if (Number.isFinite(hideStart)) {
                        typedPrevious.splitInfo = {
                            ...typedPrevious.hunkInfo,
                            startHiddenIndex: hideStart,
                            endHiddenIndex: len,
                            plainText: typedPrevious.text,
                        };
                        hideStart = Infinity;
                    }
                    this.#splitHunksLines = {
                        ...this.#splitHunksLines,
                        [len]: previous,
                    };
                }
            }
        }
        this.splitLineLength = this.#splitRightLines.length;
        this.#splitLastStartIndex = hideStart;
        this.#hasBuildSplit = true;
        this.notifyAll();
    }
    buildUnifiedDiffLines() {
        if (this.#hasBuildUnified)
            return;
        let oldFileLineNumber = 1;
        let newFileLineNumber = 1;
        let prevIsHidden = false;
        let hideStart = Infinity;
        const maxOldFileLineNumber = this.#oldFileResult?.maxLineNumber || 0;
        const maxNewFileLineNumber = this.#newFileResult?.maxLineNumber || 0;
        while (oldFileLineNumber <= maxOldFileLineNumber || newFileLineNumber <= maxNewFileLineNumber) {
            const oldRawLine = this.#getOldRawLine(oldFileLineNumber);
            const oldDiffLine = this.#getOldDiffLine(oldFileLineNumber);
            const newRawLine = this.#getNewRawLine(newFileLineNumber);
            const newDiffLine = this.#getNewDiffLine(newFileLineNumber);
            const oldLineHasChange = oldDiffLine?.isIncludeableLine();
            const newLineHasChange = newDiffLine?.isIncludeableLine();
            const len = this.#unifiedLines.length;
            const isHidden = !oldDiffLine && !newDiffLine;
            if (!oldRawLine && !newRawLine && !newDiffLine && !oldDiffLine)
                break;
            if (!oldLineHasChange && !newLineHasChange) {
                this.#unifiedLines.push({
                    oldLineNumber: oldFileLineNumber++,
                    newLineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                });
            }
            else if (oldLineHasChange) {
                this.#unifiedLines.push({
                    oldLineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                });
            }
            else if (newLineHasChange) {
                this.#unifiedLines.push({
                    newLineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                });
            }
            if (!prevIsHidden && isHidden) {
                hideStart = len;
            }
            prevIsHidden = isHidden;
            if (oldDiffLine && newDiffLine && !oldLineHasChange && !newLineHasChange) {
                const current = newDiffLine;
                const previous = current.index ? this.#diffLines?.[current.index - 1] : undefined;
                if (previous && previous.type === DiffLineType.Hunk) {
                    const typedPrevious = previous;
                    if (Number.isFinite(hideStart)) {
                        typedPrevious.unifiedInfo = {
                            ...typedPrevious.hunkInfo,
                            startHiddenIndex: hideStart,
                            endHiddenIndex: len,
                            plainText: typedPrevious.text,
                        };
                        hideStart = Infinity;
                    }
                    this.#unifiedHunksLines = {
                        ...this.#unifiedHunksLines,
                        [len]: previous,
                    };
                }
            }
        }
        this.unifiedLineLength = this.#unifiedLines.length;
        this.#unifiedLastStartIndex = hideStart;
        this.#hasBuildUnified = true;
        this.notifyAll();
    }
    getSplitLeftLine = (index) => {
        return this.#splitLeftLines[index];
    };
    getSplitRightLine = (index) => {
        return this.#splitRightLines[index];
    };
    getSplitHunkLine = (index) => {
        return this.#splitHunksLines?.[index];
    };
    onSplitHunkExpand = (dir, index) => {
        const current = this.#splitHunksLines?.[index];
        if (!current)
            return;
        if (dir === "all") {
            for (let i = current.splitInfo.startHiddenIndex; i < current.splitInfo.endHiddenIndex; i++) {
                const leftLine = this.#splitLeftLines[i];
                const rightLine = this.#splitRightLines[i];
                if (leftLine?.isHidden)
                    leftLine.isHidden = false;
                if (rightLine?.isHidden)
                    rightLine.isHidden = false;
            }
            current.splitInfo.plainText = current.text;
            current.splitInfo = {
                ...current.splitInfo,
                ...current.hunkInfo,
                plainText: current.text,
                startHiddenIndex: current.splitInfo.endHiddenIndex,
            };
        }
        else if (dir === "down") {
            for (let i = current.splitInfo.startHiddenIndex; i < current.splitInfo.startHiddenIndex + composeLen; i++) {
                const leftLine = this.#splitLeftLines[i];
                const rightLine = this.#splitRightLines[i];
                if (leftLine?.isHidden)
                    leftLine.isHidden = false;
                if (rightLine?.isHidden)
                    rightLine.isHidden = false;
            }
            current.splitInfo = {
                ...current.splitInfo,
                startHiddenIndex: current.splitInfo.startHiddenIndex + composeLen,
                plainText: `@@ -${current.splitInfo.oldStartIndex},${current.splitInfo.oldLength} +${current.splitInfo.newStartIndex},${current.splitInfo.newLength}`,
            };
        }
        else {
            for (let i = current.splitInfo.endHiddenIndex - composeLen; i < current.splitInfo.endHiddenIndex; i++) {
                const leftLine = this.#splitLeftLines[i];
                const rightLine = this.#splitRightLines[i];
                if (leftLine?.isHidden)
                    leftLine.isHidden = false;
                if (rightLine?.isHidden)
                    rightLine.isHidden = false;
            }
            const oldStartIndex = current.splitInfo.oldStartIndex - composeLen;
            const oldLength = current.splitInfo.oldLength + composeLen;
            const newStartIndex = current.splitInfo.newStartIndex - composeLen;
            const newLength = current.splitInfo.newLength + composeLen;
            current.splitInfo = {
                ...current.splitInfo,
                endHiddenIndex: current.splitInfo.endHiddenIndex - composeLen,
                oldStartIndex,
                oldLength,
                newStartIndex,
                newLength,
                plainText: `@@ -${oldStartIndex},${oldLength} +${newStartIndex},${newLength}`,
            };
            delete this.#splitHunksLines?.[index];
            this.#splitHunksLines[current.splitInfo.endHiddenIndex] = current;
        }
        this.notifyAll();
    };
    onSplitLastExpand = (expandAll) => {
        if (!this.#splitLastStartIndex || !Number.isFinite(this.#splitLastStartIndex))
            return;
        const start = this.#splitLastStartIndex;
        const end = expandAll ? this.splitLineLength : this.#splitLastStartIndex + composeLen;
        for (let i = start; i < end; i++) {
            const leftLine = this.#splitLeftLines[i];
            const rightLine = this.#splitRightLines[i];
            if (leftLine?.isHidden)
                leftLine.isHidden = false;
            if (rightLine?.isHidden)
                rightLine.isHidden = false;
        }
        this.#splitLastStartIndex = end;
        this.#splitLastStartIndex =
            this.#splitLastStartIndex >= this.splitLineLength ? Infinity : this.#splitLastStartIndex;
        this.notifyAll();
    };
    getUnifiedLine = (index) => {
        return this.#unifiedLines[index];
    };
    getUnifiedHunkLine = (index) => {
        return this.#unifiedHunksLines?.[index];
    };
    onUnifiedHunkExpand = (dir, index) => {
        const current = this.#unifiedHunksLines?.[index];
        if (!current)
            return;
        if (dir === "all") {
            for (let i = current.unifiedInfo.startHiddenIndex; i < current.unifiedInfo.endHiddenIndex; i++) {
                const unifiedLine = this.#unifiedLines?.[i];
                if (unifiedLine?.isHidden) {
                    unifiedLine.isHidden = false;
                }
            }
            current.unifiedInfo.plainText = current.text;
            current.unifiedInfo = {
                ...current.unifiedInfo,
                ...current.hunkInfo,
                plainText: current.text,
                startHiddenIndex: current.unifiedInfo.endHiddenIndex,
            };
        }
        else if (dir === "down") {
            for (let i = current.unifiedInfo.startHiddenIndex; i < current.unifiedInfo.startHiddenIndex + composeLen; i++) {
                const unifiedLine = this.#unifiedLines[i];
                if (unifiedLine?.isHidden)
                    unifiedLine.isHidden = false;
            }
            current.unifiedInfo = {
                ...current.unifiedInfo,
                startHiddenIndex: current.unifiedInfo.startHiddenIndex + composeLen,
                plainText: `@@ -${current.unifiedInfo.oldStartIndex},${current.unifiedInfo.oldLength} +${current.unifiedInfo.newStartIndex},${current.unifiedInfo.newLength}`,
            };
        }
        else {
            for (let i = current.unifiedInfo.endHiddenIndex - composeLen; i < current.unifiedInfo.endHiddenIndex; i++) {
                const unifiedLine = this.#unifiedLines[i];
                if (unifiedLine?.isHidden)
                    unifiedLine.isHidden = false;
            }
            const oldStartIndex = current.unifiedInfo.oldStartIndex - composeLen;
            const oldLength = current.unifiedInfo.oldLength + composeLen;
            const newStartIndex = current.unifiedInfo.newStartIndex - composeLen;
            const newLength = current.unifiedInfo.newLength + composeLen;
            current.unifiedInfo = {
                ...current.unifiedInfo,
                endHiddenIndex: current.unifiedInfo.endHiddenIndex - composeLen,
                oldStartIndex,
                oldLength,
                newStartIndex,
                newLength,
                plainText: `@@ -${oldStartIndex},${oldLength} +${newStartIndex},${newLength}`,
            };
            delete this.#unifiedHunksLines?.[index];
            this.#unifiedHunksLines[current.unifiedInfo.endHiddenIndex] = current;
        }
        this.notifyAll();
    };
    onUnifiedLastExpand = (expandAll) => {
        if (!this.#unifiedLastStartIndex || !Number.isFinite(this.#unifiedLastStartIndex))
            return;
        const start = this.#unifiedLastStartIndex;
        const end = expandAll ? this.unifiedLineLength : this.#unifiedLastStartIndex + composeLen;
        for (let i = start; i < end; i++) {
            const unifiedLine = this.#unifiedLines[i];
            if (unifiedLine?.isHidden)
                unifiedLine.isHidden = false;
        }
        this.#unifiedLastStartIndex = end;
        this.#unifiedLastStartIndex =
            this.#unifiedLastStartIndex >= this.unifiedLineLength ? Infinity : this.#unifiedLastStartIndex;
        this.notifyAll();
    };
    getOldSyntaxLine = (lineNumber) => {
        return this.#oldFileSyntaxLines?.[lineNumber];
    };
    getNewSyntaxLine = (lineNumber) => {
        return this.#newFileSyntaxLines?.[lineNumber];
    };
    subscribe = (listener) => {
        this.#listeners.push(listener);
        return () => {
            this.#listeners.filter((i) => i !== listener);
        };
    };
    notifyAll = () => {
        this.#updateCount++;
        this.#listeners.forEach((f) => f());
    };
    getUpdateCount = () => this.#updateCount;
    getNeedShowExpandAll = (mode) => {
        if (mode === "split") {
            return this.#splitLastStartIndex && Number.isFinite(this.#splitLastStartIndex);
        }
        else {
            return this.#unifiedLastStartIndex && Number.isFinite(this.#unifiedLastStartIndex);
        }
    };
    getExpandEnabled = () => !this.#composeByDiff;
    getBundle = () => {
        // common
        const hasInitRaw = this.#hasInitRaw;
        const hasInitSyntax = this.#hasInitSyntax;
        const hasBuildSplit = this.#hasBuildSplit;
        const hasBuildUnified = this.#hasBuildUnified;
        const oldFileLines = this.#oldFileLines;
        const oldFileDiffLines = this.#oldFileDiffLines;
        const oldFileSyntaxLines = this.#oldFileSyntaxLines;
        const newFileLines = this.#newFileLines;
        const newFileDiffLines = this.#newFileDiffLines;
        const newFileSyntaxLines = this.#newFileSyntaxLines;
        const splitLineLength = this.splitLineLength;
        const unifiedLineLength = this.unifiedLineLength;
        const composeByDiff = this.#composeByDiff;
        // split
        const splitLeftLines = this.#splitLeftLines;
        const splitRightLines = this.#splitRightLines;
        const splitHunkLines = this.#splitHunksLines;
        const splitLastStartIndex = this.#splitLastStartIndex;
        // unified
        const unifiedLines = this.#unifiedLines;
        const unifiedHunkLines = this.#unifiedHunksLines;
        const unifiedLastStartIndex = this.#unifiedLastStartIndex;
        return {
            hasInitRaw,
            hasInitSyntax,
            hasBuildSplit,
            hasBuildUnified,
            oldFileLines,
            oldFileDiffLines,
            oldFileSyntaxLines,
            newFileLines,
            newFileDiffLines,
            newFileSyntaxLines,
            splitLineLength,
            unifiedLineLength,
            splitLeftLines,
            splitRightLines,
            splitHunkLines,
            splitLastStartIndex,
            unifiedLines,
            unifiedHunkLines,
            unifiedLastStartIndex,
            composeByDiff,
        };
    };
    mergeBundle = (data) => {
        this.#hasInitRaw = data.hasInitRaw;
        this.#hasInitSyntax = data.hasInitSyntax;
        this.#hasBuildSplit = data.hasBuildSplit;
        this.#hasBuildUnified = data.hasBuildUnified;
        this.#composeByDiff = data.composeByDiff;
        this.#oldFileLines = data.oldFileLines;
        this.#oldFileDiffLines = data.oldFileDiffLines;
        this.#oldFileSyntaxLines = data.oldFileSyntaxLines;
        this.#newFileLines = data.newFileLines;
        this.#newFileDiffLines = data.newFileDiffLines;
        this.#newFileSyntaxLines = data.newFileSyntaxLines;
        this.splitLineLength = data.splitLineLength;
        this.unifiedLineLength = data.unifiedLineLength;
        this.#splitLeftLines = data.splitLeftLines;
        this.#splitRightLines = data.splitRightLines;
        this.#splitHunksLines = data.splitHunkLines;
        this.#splitLastStartIndex = data.splitLastStartIndex;
        this.#unifiedLines = data.unifiedLines;
        this.#unifiedHunksLines = data.unifiedHunkLines;
        this.#unifiedLastStartIndex = data.unifiedLastStartIndex;
        this.notifyAll();
    };
    _getFullBundle = () => {
        const bundle = this.getBundle();
        const oldFileResult = this.#oldFileResult;
        const newFileResult = this.#newFileResult;
        const diffLines = this.#diffLines;
        const diffListResults = this.#diffListResults;
        return {
            ...bundle,
            oldFileResult,
            newFileResult,
            diffLines,
            diffListResults,
        };
    };
    _mergeFullBundle = (data) => {
        this.mergeBundle(data);
        this.#oldFileResult = data.oldFileResult;
        this.#newFileResult = data.newFileResult;
        this.#diffLines = data.diffLines;
        this.#diffListResults = data.diffListResults;
    };
}

var DiffFileLineType;
(function (DiffFileLineType) {
    DiffFileLineType[DiffFileLineType["hunk"] = 1] = "hunk";
    DiffFileLineType[DiffFileLineType["content"] = 2] = "content";
    DiffFileLineType[DiffFileLineType["widget"] = 3] = "widget";
    DiffFileLineType[DiffFileLineType["extend"] = 4] = "extend";
    DiffFileLineType[DiffFileLineType["lastHunk"] = 5] = "lastHunk";
})(DiffFileLineType || (DiffFileLineType = {}));
const getSplitLines = (diffFile, options) => {
    const splitLineLength = diffFile.splitLineLength;
    const splitLines = [];
    numIterator(splitLineLength, (index) => {
        const hunkLine = diffFile.getSplitHunkLine(index);
        const splitLeftLine = diffFile.getSplitLeftLine(index);
        const splitRightLine = diffFile.getSplitRightLine(index);
        const widgetLine = options?.hasRenderWidget && options.widgetData?.[index + 1];
        const extendLine = options?.hasRenderExtend && options.extendData?.[index + 1];
        hunkLine &&
            hunkLine.splitInfo &&
            hunkLine.splitInfo.startHiddenIndex < hunkLine.splitInfo.endHiddenIndex &&
            splitLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        !splitLeftLine?.isHidden &&
            !splitRightLine?.isHidden &&
            splitLines.push({
                type: DiffFileLineType.content,
                index,
                lineNumber: index + 1,
                splitLine: { left: splitLeftLine, right: splitRightLine },
            });
        widgetLine &&
            splitLines.push({ type: DiffFileLineType.widget, index, lineNumber: index + 1, widgetLine: widgetLine });
        extendLine &&
            splitLines.push({ type: DiffFileLineType.extend, index, lineNumber: index + 1, extendLine: extendLine });
    });
    const lastHunkLine = diffFile.getNeedShowExpandAll("split");
    const expandEnabled = diffFile.getExpandEnabled();
    if (lastHunkLine && expandEnabled) {
        splitLines.push({
            type: DiffFileLineType.lastHunk,
            index: splitLineLength,
            lineNumber: splitLineLength + 1,
        });
    }
    return splitLines;
};
const getSplitContentLines = (diffFile) => {
    const lines = getSplitLines(diffFile, {});
    return lines.filter((line) => line.type === DiffFileLineType.content);
};
const getUnifiedLines = (diffFile, options) => {
    const unifiedLineLength = diffFile.unifiedLineLength;
    const unifiedLines = [];
    numIterator(unifiedLineLength, (index) => {
        const hunkLine = diffFile.getUnifiedHunkLine(index);
        const unifiedLine = diffFile.getUnifiedLine(index);
        const widgetLine = options?.hasRenderWidget && options.widgetData?.[index + 1];
        const extendLine = options?.hasRenderExtend && options.extendData?.[index + 1];
        hunkLine &&
            hunkLine.unifiedInfo &&
            hunkLine.unifiedInfo.startHiddenIndex < hunkLine.unifiedInfo.endHiddenIndex &&
            unifiedLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        !unifiedLine.isHidden &&
            unifiedLines.push({ type: DiffFileLineType.content, index, lineNumber: index + 1, unifiedLine: unifiedLine });
        widgetLine &&
            unifiedLines.push({ type: DiffFileLineType.widget, index, lineNumber: index + 1, widgetLine: widgetLine });
        extendLine &&
            unifiedLines.push({ type: DiffFileLineType.extend, index, lineNumber: index + 1, extendLine: extendLine });
    });
    const lastHunkLine = diffFile.getNeedShowExpandAll("unified");
    const expandEnabled = diffFile.getExpandEnabled();
    if (lastHunkLine && expandEnabled) {
        unifiedLines.push({
            type: DiffFileLineType.lastHunk,
            index: unifiedLineLength,
            lineNumber: unifiedLineLength + 1,
        });
    }
    return unifiedLines;
};
const getUnifiedContentLine = (diffFile) => {
    const lines = getUnifiedLines(diffFile, {});
    return lines.filter((line) => line.type === DiffFileLineType.content);
};

const useUnmount = (cb, deps) => {
    const ref = React.useRef(cb);
    ref.current = cb;
    React.useEffect(() => ref.current, deps);
};

const useDomWidth = ({ selector, enable }) => {
    const [width, setWidth] = React.useState(0);
    const { useDiffContext } = useDiffViewContext();
    const id = useDiffContext(React.useCallback((s) => s.id, []));
    React.useEffect(() => {
        if (enable) {
            const container = document.querySelector(`#diff-root${id}`);
            const wrapper = container?.querySelector(selector);
            if (!wrapper)
                return;
            const typedWrapper = wrapper;
            const cb = () => {
                const rect = wrapper?.getBoundingClientRect();
                setWidth(rect?.width ?? 0);
            };
            cb();
            const cleanCb = () => {
                typedWrapper.__observeCallback.delete(cb);
                if (typedWrapper.__observeCallback.size === 0) {
                    typedWrapper.__observeInstance?.disconnect();
                    typedWrapper.removeAttribute("data-observe");
                    delete typedWrapper.__observeCallback;
                    delete typedWrapper.__observeInstance;
                }
            };
            if (typedWrapper.__observeCallback) {
                typedWrapper.__observeCallback.add(cb);
                return () => cleanCb();
            }
            typedWrapper.__observeCallback = new Set();
            typedWrapper.__observeCallback.add(cb);
            const observer = new ResizeObserver(() => typedWrapper.__observeCallback.forEach((cb) => cb()));
            typedWrapper.__observeInstance = observer;
            observer.observe(typedWrapper);
            typedWrapper.setAttribute("data-observe", "height");
            return () => cleanCb();
        }
    }, [selector, enable, id]);
    return width;
};

exports.DiffModeEnum = void 0;
(function (DiffModeEnum) {
    DiffModeEnum[DiffModeEnum["Split"] = 1] = "Split";
    DiffModeEnum[DiffModeEnum["Unified"] = 2] = "Unified";
})(exports.DiffModeEnum || (exports.DiffModeEnum = {}));
const DiffViewContext = React.createContext(null);
DiffViewContext.displayName = "DiffViewContext";
const useDiffViewContext = () => React.useContext(DiffViewContext);

const isClient = typeof window !== "undefined";
const useSafeLayout = isClient ? React.useLayoutEffect : React.useEffect;

const useSyncHeight = ({ selector, side, enable, }) => {
    const { useDiffContext } = useDiffViewContext();
    const id = useDiffContext(React.useCallback((s) => s.id, []));
    useSafeLayout(() => {
        if (enable) {
            const container = document.querySelector(`#diff-root${id}`);
            const elements = Array.from(container?.querySelectorAll(selector) || []);
            if (elements.length === 2) {
                const ele1 = elements[0];
                const ele2 = elements[1];
                const cb = () => {
                    ele1.style.height = "auto";
                    ele2.style.height = "auto";
                    const rect1 = ele1.getBoundingClientRect();
                    const rect2 = ele2.getBoundingClientRect();
                    if (rect1.height !== rect2.height) {
                        const maxHeight = Math.max(rect1.height, rect2.height);
                        ele1.style.height = maxHeight + "px";
                        ele2.style.height = maxHeight + "px";
                        ele1.setAttribute("data-sync-height", String(maxHeight));
                        ele2.setAttribute("data-sync-height", String(maxHeight));
                    }
                    else {
                        ele1.removeAttribute("data-sync-height");
                        ele2.removeAttribute("data-sync-height");
                    }
                };
                cb();
                const observer = new ResizeObserver(cb);
                const target = ele1.getAttribute("data-side") === side ? ele1 : ele2;
                observer.observe(target);
                target.setAttribute("data-observe", "height");
                return () => {
                    observer.disconnect();
                    target?.removeAttribute("data-observe");
                };
            }
        }
    }, [selector, enable, side, id]);
};

const addContentBGName = "--diff-add-content--";
const delContentBGName = "--diff-del-content--";
const addLineNumberBGName = "--diff-add-lineNumber--";
const delLineNumberBGName = "--diff-del-lineNumber--";
const plainContentBGName = "--diff-plain-content--";
const expandContentBGName = "--diff-expand-content--";
const plainLineNumberColorName = "--diff-plain-lineNumber-color--";
const plainLineNumberBGName = "--diff-plain-lineNumber--";
const hunkContentBGName = "--diff-hunk-content--";
const hunkContentColorName = "--diff-hunk-content-color--";
const hunkLineNumberBGName = "--diff-hunk-lineNumber--";
const addContentHighlightBGName = "--diff-add-content-highlight--";
const delContentHighlightBGName = "--diff-del-content-highlight--";
const addWidgetBGName = "--diff-add-widget--";
const addWidgetColorName = "--diff-add-widget-color--";
const emptyBGName = "--diff-empty-content--";
const getContentBG = (isAdded, isDelete, hasDiff) => {
    return isAdded
        ? `var(${addContentBGName})`
        : isDelete
            ? `var(${delContentBGName})`
            : hasDiff
                ? `var(${plainContentBGName})`
                : `var(${expandContentBGName})`;
};
const getLineNumberBG = (isAdded, isDelete, hasDiff) => {
    return isAdded
        ? `var(${addLineNumberBGName})`
        : isDelete
            ? `var(${delLineNumberBGName})`
            : hasDiff
                ? `var(${plainLineNumberBGName})`
                : `var(${expandContentBGName})`;
};

const _DiffSplitExtendLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const { extendData, renderExtendLine } = useDiffContext(React__namespace.useCallback((s) => ({ extendData: s.extendData, renderExtendLine: s.renderExtendLine }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineExtend = extendData?.oldFile?.[oldLine?.lineNumber];
    const newLineExtend = extendData?.newFile?.[newLine?.lineNumber];
    const currentExtend = side === exports.SplitSide.old ? oldLineExtend : newLineExtend;
    const currentLineNumber = side === exports.SplitSide.old ? oldLine.lineNumber : newLine.lineNumber;
    const otherSide = side === exports.SplitSide.old ? exports.SplitSide.new : exports.SplitSide.old;
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-extend"]`,
        side: currentExtend ? exports.SplitSide[side] : exports.SplitSide[otherSide],
        enable: side === exports.SplitSide.new && typeof renderExtendLine === "function",
    });
    const width = useDomWidth({
        selector: side === exports.SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper",
        enable: !!currentExtend && typeof renderExtendLine === "function",
    });
    if (!renderExtendLine)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", "data-side": exports.SplitSide[side], className: "diff-line diff-line-extend" }, currentExtend ? (React__namespace.createElement("td", { className: `diff-line-extend-${exports.SplitSide[side]}-content p-0`, colSpan: 2 },
        React__namespace.createElement("div", { className: "diff-line-extend-wrapper sticky left-0", style: { width } }, width > 0 &&
            renderExtendLine?.({
                diffFile,
                side,
                lineNumber: currentLineNumber,
                data: currentExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React__namespace.createElement("td", { className: `diff-line-extend-${exports.SplitSide[side]}-placeholder p-0 select-none`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitExtendLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const hasExtend = useDiffContext(React__namespace.useCallback((s) => s.extendData?.oldFile?.[oldLine?.lineNumber] || s.extendData?.newFile?.[newLine?.lineNumber], [oldLine?.lineNumber, newLine?.lineNumber]));
    // if the expand action not enabled, the `isHidden` property will never change
    const enableExpand = diffFile.getExpandEnabled();
    const currentLine = side === exports.SplitSide.old ? oldLine : newLine;
    const currentIsShow = hasExtend && (!currentLine.isHidden || !enableExpand);
    if (!currentIsShow)
        return null;
    return React__namespace.createElement(_DiffSplitExtendLine$1, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber });
};

const ExpandDown = ({ className }) => {
    return (React__namespace.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React__namespace.createElement("path", { d: "m8.177 14.323 2.896-2.896a.25.25 0 0 0-.177-.427H8.75V7.764a.75.75 0 1 0-1.5 0V11H5.104a.25.25 0 0 0-.177.427l2.896 2.896a.25.25 0 0 0 .354 0ZM2.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75ZM8.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z" })));
};
const ExpandUp = ({ className }) => {
    return (React__namespace.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React__namespace.createElement("path", { d: "M7.823 1.677 4.927 4.573A.25.25 0 0 0 5.104 5H7.25v3.236a.75.75 0 1 0 1.5 0V5h2.146a.25.25 0 0 0 .177-.427L8.177 1.677a.25.25 0 0 0-.354 0ZM13.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Zm-3.75.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM7.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5ZM4 11.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM1.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Z" })));
};
const ExpandAll = ({ className }) => {
    return (React__namespace.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React__namespace.createElement("path", { d: "m8.177.677 2.896 2.896a.25.25 0 0 1-.177.427H8.75v1.25a.75.75 0 0 1-1.5 0V4H5.104a.25.25 0 0 1-.177-.427L7.823.677a.25.25 0 0 1 .354 0ZM7.25 10.75a.75.75 0 0 1 1.5 0V12h2.146a.25.25 0 0 1 .177.427l-2.896 2.896a.25.25 0 0 1-.354 0l-2.896-2.896A.25.25 0 0 1 5.104 12H7.25v-1.25Zm-5-2a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 6 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 12 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z" })));
};

const _DiffSplitHunkLine = ({ index, diffFile, side, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-hunk"]`,
        side: exports.SplitSide[exports.SplitSide.old],
        enable: side === exports.SplitSide.new,
    });
    const enableHunkAction = side === exports.SplitSide.old;
    const isExpandAll = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.endHiddenIndex - currentHunk.splitInfo.startHiddenIndex < composeLen;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", "data-side": exports.SplitSide[side], className: "diff-line diff-line-hunk" }, enableHunkAction ? (React__namespace.createElement(React__namespace.Fragment, null,
        React__namespace.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, expandEnabled ? (isFirstLine ? (React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
            React__namespace.createElement(ExpandUp, { className: "fill-current" }))) : isExpandAll ? (React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onSplitHunkExpand("all", index) },
            React__namespace.createElement(ExpandAll, { className: "fill-current" }))) : (React__namespace.createElement(React__namespace.Fragment, null,
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
                React__namespace.createElement(ExpandDown, { className: "fill-current" })),
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                React__namespace.createElement(ExpandUp, { className: "fill-current" }))))) : (React__namespace.createElement("span", null, "\u2002"))),
        React__namespace.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React__namespace.createElement("div", { className: "pl-[1.5em]", style: {
                    color: `var(${hunkContentColorName})`,
                } }, currentHunk.splitInfo.plainText)))) : (React__namespace.createElement("td", { className: "diff-line-hunk-placeholder select-none", colSpan: 2, style: { backgroundColor: `var(${hunkContentBGName})` } },
        React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitHunkLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const currentIsShow = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.startHiddenIndex < currentHunk.splitInfo.endHiddenIndex;
    if (!currentIsShow)
        return null;
    return React__namespace.createElement(_DiffSplitHunkLine, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber });
};
const _DiffSplitLastHunkLine = ({ diffFile, side }) => {
    const enableHunkAction = side === exports.SplitSide.old;
    useSyncHeight({
        selector: `tr[data-line="last-hunk"]`,
        side: exports.SplitSide[exports.SplitSide.old],
        enable: side === exports.SplitSide.new,
    });
    return (React__namespace.createElement("tr", { "data-line": "last-hunk", "data-state": "hunk", "data-side": exports.SplitSide[side], className: "diff-line diff-line-hunk" }, enableHunkAction ? (React__namespace.createElement(React__namespace.Fragment, null,
        React__namespace.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } },
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px] relative", title: "Expand Down", "data-title": "Expand Down", onClick: () => {
                    diffFile.onSplitLastExpand();
                } },
                React__namespace.createElement(ExpandDown, { className: "fill-current" }))),
        React__namespace.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle select-none", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React__namespace.createElement("span", null, "\u2002")))) : (React__namespace.createElement("td", { className: "diff-line-hunk-placeholder select-none", colSpan: 2, style: { backgroundColor: `var(${hunkContentBGName})` } },
        React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitLastHunkLine$1 = ({ diffFile, side }) => {
    const currentIsShow = diffFile.getNeedShowExpandAll("split");
    const expandEnabled = diffFile.getExpandEnabled();
    if (!currentIsShow || !expandEnabled)
        return null;
    return React__namespace.createElement(_DiffSplitLastHunkLine, { diffFile: diffFile, side: side });
};

const DiffSplitAddWidget = ({ side, className, lineNumber, onWidgetClick, onOpenAddWidget, }) => {
    return (React__namespace.createElement("div", { className: "diff-add-widget-wrapper" + (className ? " " + className : ""), style: {
            width: "calc(var(--diff-font-size--) * 1.4)",
            height: "calc(var(--diff-font-size--) * 1.4)",
        } },
        React__namespace.createElement("button", { className: "diff-add-widget w-0 h-0 overflow-hidden cursor-pointer rounded-md flex items-center justify-center transition-transform origin-center group-hover:w-full group-hover:h-full hover:scale-110", style: {
                color: `var(${addWidgetColorName})`,
                zIndex: 1,
                fontSize: `1.2em`,
                backgroundColor: `var(${addWidgetBGName})`,
            }, onClick: () => {
                onOpenAddWidget(lineNumber, side);
                onWidgetClick?.(lineNumber, side);
            } }, "+")));
};
const DiffUnifiedAddWidget = ({ lineNumber, side, onWidgetClick, onOpenAddWidget, }) => {
    return (React__namespace.createElement("div", { className: "diff-add-widget-wrapper absolute left-[100%] top-[1px] translate-x-[-50%]", style: {
            width: "calc(var(--diff-font-size--) * 1.4)",
            height: "calc(var(--diff-font-size--) * 1.4)",
        } },
        React__namespace.createElement("button", { className: "diff-add-widget overflow-hidden cursor-pointer rounded-md w-0 h-0 flex items-center justify-center transition-transform origin-center group-hover:w-full group-hover:h-full hover:scale-110", style: {
                color: `var(${addWidgetColorName})`,
                zIndex: 1,
                fontSize: `1.2em`,
                backgroundColor: `var(${addWidgetBGName})`,
            }, onClick: () => {
                onOpenAddWidget(lineNumber, side);
                onWidgetClick?.(lineNumber, side);
            } }, "+")));
};

const DiffString = ({ rawLine, diffLine, operator, }) => {
    const range = diffLine?.range;
    if (range && range.length > 0 && range.length < rawLine.length) {
        const str1 = rawLine.slice(0, range.location);
        const str2 = rawLine.slice(range.location, range.location + range.length);
        const str3 = rawLine.slice(range.location + range.length);
        return (React__namespace.createElement("span", { className: "diff-line-content-raw" },
            React__namespace.createElement("span", { "data-range-start": range.location, "data-range-end": range.location + range.length },
                str1,
                React__namespace.createElement("span", { "data-diff-highlight": true, className: "rounded-[0.2em]", style: {
                        backgroundColor: operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`,
                    } }, str2 === "\r" ? "␍" : str2 === "\n" ? "␊" : str2 === "\r\n" ? "␍␊" : str2),
                str3)));
    }
    return React__namespace.createElement("span", { className: "diff-line-content-raw" }, rawLine);
};
const DiffSyntax = ({ rawLine, diffLine, operator, syntaxLine, }) => {
    if (!syntaxLine) {
        return React__namespace.createElement(DiffString, { rawLine: rawLine, diffLine: diffLine, operator: operator });
    }
    const range = diffLine?.range;
    if (range && range.length > 0 && range.length < syntaxLine.valueLength) {
        return (React__namespace.createElement("span", { className: "diff-line-syntax-raw" },
            React__namespace.createElement("span", { "data-range-start": range.location, "data-range-end": range.location + range.length }, syntaxLine.nodeList?.map(({ node, wrapper }, index) => {
                if (node.endIndex < range.location || range.location + range.length < node.startIndex) {
                    return (React__namespace.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: wrapper?.properties?.className?.join(" ") }, node.value));
                }
                else {
                    const index1 = range.location - node.startIndex;
                    const index2 = index1 < 0 ? 0 : index1;
                    const str1 = node.value.slice(0, index2);
                    const str2 = node.value.slice(index2, index1 + range.length);
                    const str3 = node.value.slice(index1 + range.length);
                    const isStart = str1.length || range.location === node.startIndex;
                    const isEnd = str3.length || node.endIndex === range.location + range.length - 1;
                    return (React__namespace.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: wrapper?.properties?.className?.join(" ") },
                        str1,
                        React__namespace.createElement("span", { "data-diff-highlight": true, style: {
                                backgroundColor: operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`,
                                borderTopLeftRadius: isStart ? "0.2em" : undefined,
                                borderBottomLeftRadius: isStart ? "0.2em" : undefined,
                                borderTopRightRadius: isEnd ? "0.2em" : undefined,
                                borderBottomRightRadius: isEnd ? "0.2em" : undefined,
                            } }, str2 === "\r" ? "␍" : str2 === "\n" ? "␊" : str2 === "\r\n" ? "␍␊" : str2),
                        str3));
                }
            }))));
    }
    return (React__namespace.createElement("span", { className: "diff-line-syntax-raw" }, syntaxLine?.nodeList?.map(({ node, wrapper }, index) => (React__namespace.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: wrapper?.properties?.className?.join(" ") }, node.value)))));
};
const DiffContent = ({ diffLine, rawLine, syntaxLine, enableWrap, enableHighlight, }) => {
    const isAdded = diffLine?.type === DiffLineType.Add;
    const isDelete = diffLine?.type === DiffLineType.Delete;
    return (React__namespace.createElement("div", { className: "diff-line-content-item pl-[2.0em]", "data-val": rawLine, style: {
            whiteSpace: enableWrap ? "pre-wrap" : "pre",
            wordBreak: enableWrap ? "break-all" : "initial",
        } },
        React__namespace.createElement("span", { "data-operator": isAdded ? "+" : isDelete ? "-" : undefined, className: "diff-line-content-operator inline-block w-[1.5em] ml-[-1.5em] indent-[0.2em] select-none" }, isAdded ? "+" : isDelete ? "-" : " "),
        enableHighlight && syntaxLine ? (React__namespace.createElement(DiffSyntax, { operator: isAdded ? "add" : isDelete ? "del" : undefined, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine })) : (React__namespace.createElement(DiffString, { operator: isAdded ? "add" : isDelete ? "del" : undefined, rawLine: rawLine, diffLine: diffLine }))));
};

const DiffWidgetContext = React.createContext(null);
DiffWidgetContext.displayName = "DiffWidgetContext";
const useDiffWidgetContext = () => React.useContext(DiffWidgetContext);

const _DiffSplitLine$1 = ({ index, diffFile, lineNumber, side, }) => {
    const getCurrentSyntaxLine = side === exports.SplitSide.old ? diffFile.getOldSyntaxLine : diffFile.getNewSyntaxLine;
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const currentLine = side === exports.SplitSide.old ? oldLine : newLine;
    const hasDiff = !!currentLine?.diff;
    const hasContent = !!currentLine.lineNumber;
    const hasChange = checkDiffLineIncludeChange(currentLine?.diff);
    const isAdded = currentLine?.diff?.type === DiffLineType.Add;
    const isDelete = currentLine?.diff?.type === DiffLineType.Delete;
    const { useDiffContext } = useDiffViewContext();
    const { enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React__namespace.useCallback((s) => ({
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const contentBG = getContentBG(isAdded, isDelete, hasDiff);
    const lineNumberBG = getLineNumberBG(isAdded, isDelete, hasDiff);
    const syntaxLine = getCurrentSyntaxLine(currentLine.lineNumber);
    return (React__namespace.createElement("tr", { "data-line": lineNumber, "data-state": hasDiff || !hasContent ? "diff" : "plain", "data-side": exports.SplitSide[side], className: "diff-line" + (hasContent ? " group" : "") }, hasContent ? (React__namespace.createElement(React__namespace.Fragment, null,
        React__namespace.createElement("td", { className: `diff-line-${exports.SplitSide[side]}-num sticky left-0 pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px]`, style: {
                backgroundColor: lineNumberBG,
                color: `var(${plainLineNumberColorName})`,
            } },
            hasDiff && enableAddWidget && (React__namespace.createElement(DiffSplitAddWidget, { index: index, lineNumber: currentLine.lineNumber, side: side, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
            React__namespace.createElement("span", { "data-line-num": currentLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, currentLine.lineNumber)),
        React__namespace.createElement("td", { className: `diff-line-${exports.SplitSide[side]}-content pr-[10px] align-top`, style: { backgroundColor: contentBG } },
            React__namespace.createElement(DiffContent, { enableWrap: false, diffFile: diffFile, rawLine: currentLine.value, diffLine: currentLine.diff, syntaxLine: syntaxLine, enableHighlight: enableHighlight })))) : (React__namespace.createElement("td", { className: `diff-line-${exports.SplitSide[side]}-placeholder select-none`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitLine$1 = ({ index, diffFile, lineNumber, side, }) => {
    const getCurrentLine = side === exports.SplitSide.old ? diffFile.getSplitLeftLine : diffFile.getSplitRightLine;
    const currentLine = getCurrentLine(index);
    if (currentLine?.isHidden)
        return null;
    return React__namespace.createElement(_DiffSplitLine$1, { index: index, diffFile: diffFile, lineNumber: lineNumber, side: side });
};

const _DiffSplitWidgetLine$1 = ({ diffFile, side, lineNumber, currentLine, setWidget, currentWidget, }) => {
    const otherSide = side === exports.SplitSide.old ? exports.SplitSide.new : exports.SplitSide.old;
    const { useDiffContext } = useDiffViewContext();
    const renderWidgetLine = useDiffContext(React__namespace.useCallback((s) => s.renderWidgetLine, []));
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-widget"]`,
        side: currentWidget ? exports.SplitSide[side] : exports.SplitSide[otherSide],
        enable: side === exports.SplitSide.new && typeof renderWidgetLine === "function",
    });
    const width = useDomWidth({
        selector: side === exports.SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper",
        enable: !!currentWidget && typeof renderWidgetLine === "function",
    });
    if (!renderWidgetLine)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", "data-side": exports.SplitSide[side], className: "diff-line diff-line-widget" }, currentWidget ? (React__namespace.createElement("td", { className: `diff-line-widget-${exports.SplitSide[side]}-content p-0`, colSpan: 2 },
        React__namespace.createElement("div", { className: "diff-line-widget-wrapper sticky left-0", style: { width } }, width > 0 &&
            renderWidgetLine?.({
                diffFile,
                side,
                lineNumber: currentLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React__namespace.createElement("td", { className: `diff-line-widget-${exports.SplitSide[side]}-placeholder p-0`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitWidgetLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide, setWidget } = useWidget(React__namespace.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === exports.SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === exports.SplitSide.new && widgetLineNumber === newLine.lineNumber;
    const currentLine = side === exports.SplitSide.old ? oldLine : newLine;
    const currentWidget = side === exports.SplitSide.old ? oldLineWidget : newLineWidget;
    const currentIsShow = oldLineWidget || newLineWidget;
    if (!currentIsShow)
        return null;
    return (React__namespace.createElement(_DiffSplitWidgetLine$1, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber, currentLine: currentLine, setWidget: setWidget, currentWidget: currentWidget }));
};

const removeAllSelection = () => {
    const selection = window.getSelection();
    for (let i = 0; i < selection.rangeCount; i++) {
        selection.removeRange(selection.getRangeAt(i));
    }
};
const syncScroll = (left, right) => {
    const onScroll = function (event) {
        if (event === null || event.target === null)
            return;
        if (event.target === left) {
            right.scrollTop = left.scrollTop;
            right.scrollLeft = left.scrollLeft;
        }
        else {
            left.scrollTop = right.scrollTop;
            left.scrollLeft = right.scrollLeft;
        }
    };
    if (!left.onscroll) {
        left.onscroll = onScroll;
    }
    if (!right.onscroll) {
        right.onscroll = onScroll;
    }
    return () => {
        left.onscroll = null;
        right.onscroll = null;
    };
};

const onMouseDown$1 = (e) => {
    const ele = e.target;
    // need remove all the selection
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
        removeAllSelection();
        return;
    }
};
const DiffSplitViewTable = ({ side, diffFile }) => {
    const className = side === exports.SplitSide.new ? "new-diff-table" : "old-diff-table";
    const lines = getSplitContentLines(diffFile);
    return (React__namespace.createElement("table", { className: className + " border-collapse w-full", "data-mode": exports.SplitSide[side] },
        React__namespace.createElement("colgroup", null,
            React__namespace.createElement("col", { className: `diff-table-${exports.SplitSide[side]}-num-col` }),
            React__namespace.createElement("col", { className: `diff-table-${exports.SplitSide[side]}-content-col` })),
        React__namespace.createElement("thead", { className: "hidden" },
            React__namespace.createElement("tr", null,
                React__namespace.createElement("th", { scope: "col" },
                    exports.SplitSide[side],
                    " line number"),
                React__namespace.createElement("th", { scope: "col" },
                    exports.SplitSide[side],
                    " line content"))),
        React__namespace.createElement("tbody", { className: "diff-table-body leading-[1.4]", onMouseDownCapture: onMouseDown$1 },
            lines.map((line) => (React__namespace.createElement(React.Fragment, { key: line.index },
                React__namespace.createElement(DiffSplitHunkLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React__namespace.createElement(DiffSplitLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React__namespace.createElement(DiffSplitWidgetLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React__namespace.createElement(DiffSplitExtendLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile })))),
            React__namespace.createElement(DiffSplitLastHunkLine$1, { side: side, diffFile: diffFile }))));
};
const DiffSplitViewNormal = React.memo(({ diffFile }) => {
    const ref1 = React.useRef(null);
    const ref2 = React.useRef(null);
    shim.useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    React.useEffect(() => {
        const left = ref1.current;
        const right = ref2.current;
        if (!left || !right)
            return;
        return syncScroll(left, right);
    }, []);
    return (React__namespace.createElement("div", { className: "split-diff-view split-diff-view-wrap w-full flex basis-[50%]" },
        React__namespace.createElement("div", { className: "old-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", ref: ref1, style: {
                overscrollBehaviorX: "none",
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React__namespace.createElement(DiffSplitViewTable, { side: exports.SplitSide.old, diffFile: diffFile })),
        React__namespace.createElement("div", { className: "diff-split-line w-[1.5px] bg-[#ccc]" }),
        React__namespace.createElement("div", { className: "new-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", ref: ref2, style: {
                overscrollBehaviorX: "none",
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React__namespace.createElement(DiffSplitViewTable, { side: exports.SplitSide.new, diffFile: diffFile }))));
});
DiffSplitViewNormal.displayName = "DiffSplitViewNormal";

let canvasCtx = null;
class TextMeasure {
    #key = "";
    #map = {};
    #getInstance() {
        canvasCtx = canvasCtx || document.createElement("canvas").getContext("2d");
        return canvasCtx;
    }
    measure(text, font) {
        const currentKey = `${font?.fontFamily}-${font?.fontStyle}-${font?.fontSize}-${text}`;
        if (this.#map[currentKey]) {
            return this.#map[currentKey];
        }
        const instance = this.#getInstance();
        if (font) {
            const currentFontKey = `${font.fontFamily}-${font.fontStyle}-${font.fontSize}`;
            if (this.#key !== currentFontKey) {
                this.#key = currentFontKey;
                instance.font = `${font.fontStyle || ""} ${font.fontSize || ""} ${font.fontFamily || ""}`;
            }
        }
        else {
            instance.font = "";
        }
        const textWidth = instance.measureText(text).width;
        return textWidth;
    }
}
const measureInstance = new TextMeasure();
const useTextWidth = ({ text, font, }) => {
    const [width, setWidth] = React.useState(0);
    useSafeLayout(() => {
        const width = measureInstance.measure(text, font);
        setWidth(width);
    }, [text, font]);
    return width;
};

const _DiffSplitExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    // 需要显示的时候才进行方法订阅，可以大幅度提高性能
    const { extendData, renderExtendLine } = useDiffContext(React__namespace.useCallback((s) => ({ extendData: s.extendData, renderExtendLine: s.renderExtendLine }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineExtend = extendData?.oldFile?.[oldLine?.lineNumber];
    const newLineExtend = extendData?.newFile?.[newLine?.lineNumber];
    if (!renderExtendLine)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", className: "diff-line diff-line-extend" },
        oldLineExtend ? (React__namespace.createElement("td", { className: "diff-line-extend-old-content p-0", colSpan: 2 },
            React__namespace.createElement("div", { className: "diff-line-extend-wrapper" }, renderExtendLine?.({
                diffFile,
                side: exports.SplitSide.old,
                lineNumber: oldLine.lineNumber,
                data: oldLineExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React__namespace.createElement("td", { className: "diff-line-extend-old-placeholder p-0 select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React__namespace.createElement("span", null, "\u2002"))),
        newLineExtend ? (React__namespace.createElement("td", { className: "diff-line-extend-new-content p-0 border-l-[1px] border-l-[#ccc]", colSpan: 2 },
            React__namespace.createElement("div", { className: "diff-line-extend-wrapper" }, renderExtendLine?.({
                diffFile,
                side: exports.SplitSide.new,
                lineNumber: newLine.lineNumber,
                data: newLineExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React__namespace.createElement("td", { className: "diff-line-extend-new-placeholder p-0 border-l-[1px] border-l-[#ccc] select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const hasExtend = useDiffContext(React__namespace.useCallback((s) => s.extendData?.oldFile?.[oldLine?.lineNumber] || s.extendData?.newFile?.[newLine?.lineNumber], [oldLine?.lineNumber, newLine?.lineNumber]));
    // if the expand action not enabled, the `isHidden` property will never change
    const enableExpand = diffFile.getExpandEnabled();
    const currentIsShow = hasExtend && ((!oldLine?.isHidden && !newLine?.isHidden) || !enableExpand);
    if (!currentIsShow)
        return null;
    return React__namespace.createElement(_DiffSplitExtendLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const DiffSplitHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    const isExpandAll = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.endHiddenIndex - currentHunk.splitInfo.startHiddenIndex < composeLen;
    const currentIsShow = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.startHiddenIndex < currentHunk.splitInfo.endHiddenIndex;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    if (!currentIsShow)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React__namespace.createElement("td", { className: "diff-line-hunk-action p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, expandEnabled &&
            (isFirstLine ? (React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                React__namespace.createElement(ExpandUp, { className: "fill-current" }))) : isExpandAll ? (React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onSplitHunkExpand("all", index) },
                React__namespace.createElement(ExpandAll, { className: "fill-current" }))) : (React__namespace.createElement(React__namespace.Fragment, null,
                React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
                    React__namespace.createElement(ExpandDown, { className: "fill-current" })),
                React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                    React__namespace.createElement(ExpandUp, { className: "fill-current" })))))),
        React__namespace.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` }, colSpan: 3 },
            React__namespace.createElement("div", { className: "pl-[1.5em]", style: {
                    color: `var(${hunkContentColorName})`,
                } }, currentHunk.splitInfo.plainText))));
};
const DiffSplitLastHunkLine = ({ diffFile }) => {
    const currentIsShow = diffFile.getNeedShowExpandAll("split");
    const expandEnabled = diffFile.getExpandEnabled();
    if (!currentIsShow || !expandEnabled)
        return null;
    return (React__namespace.createElement("tr", { "data-line": "last-hunk", "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React__namespace.createElement("td", { className: "diff-line-hunk-action p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } },
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => {
                    diffFile.onSplitLastExpand();
                } },
                React__namespace.createElement(ExpandDown, { className: "fill-current" }))),
        React__namespace.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle select-none", colSpan: 3, style: { backgroundColor: `var(${hunkContentBGName})` } },
            React__namespace.createElement("span", null, "\u2002"))));
};

const _DiffSplitLine = ({ index, diffFile, lineNumber }) => {
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldSyntaxLine = diffFile.getOldSyntaxLine(oldLine?.lineNumber);
    const newSyntaxLine = diffFile.getNewSyntaxLine(newLine?.lineNumber);
    const hasDiff = !!oldLine?.diff || !!newLine?.diff;
    const hasChange = checkDiffLineIncludeChange(oldLine?.diff) || checkDiffLineIncludeChange(newLine?.diff);
    const oldLineIsDelete = oldLine?.diff?.type === DiffLineType.Delete;
    const newLineIsAdded = newLine?.diff?.type === DiffLineType.Add;
    const { useDiffContext } = useDiffViewContext();
    const { enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React__namespace.useCallback((s) => ({
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const hasOldLine = !!oldLine.lineNumber;
    const hasNewLine = !!newLine.lineNumber;
    const oldLineContentBG = getContentBG(false, oldLineIsDelete, hasDiff);
    const oldLineNumberBG = getLineNumberBG(false, oldLineIsDelete, hasDiff);
    const newLineContentBG = getContentBG(newLineIsAdded, false, hasDiff);
    const newLineNumberBG = getLineNumberBG(newLineIsAdded, false, hasDiff);
    return (React__namespace.createElement("tr", { "data-line": lineNumber, "data-state": hasDiff ? "diff" : "plain", className: "diff-line" },
        hasOldLine ? (React__namespace.createElement(React__namespace.Fragment, null,
            React__namespace.createElement("td", { className: "diff-line-old-num group relative pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px]", "data-side": exports.SplitSide[exports.SplitSide.old], style: { backgroundColor: oldLineNumberBG, color: `var(${plainLineNumberColorName})` } },
                hasDiff && enableAddWidget && (React__namespace.createElement(DiffSplitAddWidget, { index: index, lineNumber: oldLine.lineNumber, side: exports.SplitSide.old, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React__namespace.createElement("span", { "data-line-num": oldLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, oldLine.lineNumber)),
            React__namespace.createElement("td", { className: "diff-line-old-content group relative pr-[10px] align-top", "data-side": exports.SplitSide[exports.SplitSide.old], style: { backgroundColor: oldLineContentBG } },
                hasDiff && enableAddWidget && (React__namespace.createElement(DiffSplitAddWidget, { index: index, lineNumber: oldLine.lineNumber, side: exports.SplitSide.old, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute right-[100%] translate-x-[50%] z-[1] select-none", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React__namespace.createElement(DiffContent, { enableWrap: true, diffFile: diffFile, rawLine: oldLine.value, diffLine: oldLine.diff, syntaxLine: oldSyntaxLine, enableHighlight: enableHighlight })))) : (React__namespace.createElement("td", { className: "diff-line-old-placeholder select-none", "data-side": exports.SplitSide[exports.SplitSide.old], style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React__namespace.createElement("span", null, "\u2002"))),
        hasNewLine ? (React__namespace.createElement(React__namespace.Fragment, null,
            React__namespace.createElement("td", { className: "diff-line-new-num group relative pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px] border-l-[1px] border-l-[#ccc]", "data-side": exports.SplitSide[exports.SplitSide.new], style: { backgroundColor: newLineNumberBG, color: `var(${plainLineNumberColorName})` } },
                hasDiff && enableAddWidget && (React__namespace.createElement(DiffSplitAddWidget, { index: index, lineNumber: newLine.lineNumber, side: exports.SplitSide.new, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React__namespace.createElement("span", { "data-line-num": newLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, newLine.lineNumber)),
            React__namespace.createElement("td", { className: "diff-line-new-content group relative pr-[10px] align-top", "data-side": exports.SplitSide[exports.SplitSide.new], style: { backgroundColor: newLineContentBG } },
                hasDiff && enableAddWidget && (React__namespace.createElement(DiffSplitAddWidget, { index: index, lineNumber: newLine.lineNumber, side: exports.SplitSide.new, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute right-[100%] translate-x-[50%] z-[1] select-none", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React__namespace.createElement(DiffContent, { enableWrap: true, diffFile: diffFile, rawLine: newLine.value || "", diffLine: newLine.diff, syntaxLine: newSyntaxLine, enableHighlight: enableHighlight })))) : (React__namespace.createElement("td", { className: "diff-line-new-placeholder select-none border-l-[1px] border-l-[#ccc]", style: { backgroundColor: `var(${emptyBGName})` }, "data-side": exports.SplitSide[exports.SplitSide.new], colSpan: 2 },
            React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitLine = ({ index, diffFile, lineNumber, }) => {
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    if (oldLine?.isHidden && newLine?.isHidden)
        return null;
    return React__namespace.createElement(_DiffSplitLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffSplitWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide, setWidget } = useWidget(React__namespace.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const { useDiffContext } = useDiffViewContext();
    const renderWidgetLine = useDiffContext(React__namespace.useCallback((s) => s.renderWidgetLine, []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === exports.SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === exports.SplitSide.new && widgetLineNumber === newLine.lineNumber;
    if (!renderWidgetLine)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", className: "diff-line diff-line-widget" },
        oldLineWidget ? (React__namespace.createElement("td", { className: "diff-line-widget-old-content p-0", colSpan: 2 },
            React__namespace.createElement("div", { className: "diff-line-widget-wrapper" }, renderWidgetLine?.({
                diffFile,
                side: exports.SplitSide.old,
                lineNumber: oldLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React__namespace.createElement("td", { className: "diff-line-widget-old-placeholder p-0 select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React__namespace.createElement("span", null, "\u2002"))),
        newLineWidget ? (React__namespace.createElement("td", { className: "diff-line-widget-new-content p-0 border-l-[1px] border-l-[#ccc]", colSpan: 2 },
            React__namespace.createElement("div", { className: "diff-line-widget-wrapper" }, renderWidgetLine?.({
                diffFile,
                side: exports.SplitSide.new,
                lineNumber: newLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React__namespace.createElement("td", { className: "diff-line-widget-new-placeholder p-0 border-l-[1px] border-l-[#ccc] select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React__namespace.createElement("span", null, "\u2002")))));
};
const DiffSplitWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide } = useWidget(React__namespace.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === exports.SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === exports.SplitSide.new && widgetLineNumber === newLine.lineNumber;
    const currentIsShow = oldLineWidget || newLineWidget;
    if (!currentIsShow)
        return null;
    return React__namespace.createElement(_DiffSplitWidgetLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

/* eslint-disable jsx-a11y/no-noninteractive-element-interactions */
const Style = ({ useSelector, id, }) => {
    const splitRef = useSelector((s) => s.splitRef);
    return (React__namespace.createElement("style", null, splitRef === exports.SplitSide.old
        ? `#${id} td[data-side="${exports.SplitSide[exports.SplitSide.new]}"] {user-select: none}`
        : splitRef === exports.SplitSide.new
            ? `#${id} td[data-side="${exports.SplitSide[exports.SplitSide.old]}"] {user-select: none}`
            : ""));
};
const DiffSplitViewWrap = React.memo(({ diffFile }) => {
    const splitLineLength = diffFile.splitLineLength;
    const { useDiffContext } = useDiffViewContext();
    const splitSideInfo = React.useMemo(() => reactivityStore.createStore(() => {
        const splitRef = reactivityStore.ref(undefined);
        const setSplit = (side) => {
            reactDom.flushSync(() => {
                splitRef.value = side;
            });
        };
        return { splitRef, setSplit };
    }), []);
    const setSelectSide = splitSideInfo.getReadonlyState().setSplit;
    const fontSize = useDiffContext(React.useCallback((s) => s.fontSize, []));
    shim.useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    const onMouseDown = React.useCallback((e) => {
        let ele = e.target;
        // need remove all the selection
        if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
            removeAllSelection();
            return;
        }
        while (ele && ele instanceof HTMLElement && ele.nodeName !== "TD") {
            ele = ele.parentElement;
        }
        if (ele instanceof HTMLElement) {
            const side = ele.getAttribute("data-side");
            if (side) {
                setSelectSide(exports.SplitSide[side]);
                removeAllSelection();
            }
        }
    }, [setSelectSide]);
    const width = useTextWidth({
        text: splitLineLength.toString(),
        font: { fontSize: fontSize + "px", fontFamily: "Menlo, Consolas, monospace" },
    });
    const lines = getSplitContentLines(diffFile);
    return (React__namespace.createElement("div", { className: "split-diff-view split-diff-view-normal w-full" },
        React__namespace.createElement("div", { className: "diff-table-wrapper w-full", style: {
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React__namespace.createElement(Style, { useSelector: splitSideInfo, id: `diff-root${diffFile.getId()}` }),
            React__namespace.createElement("table", { className: "diff-table border-collapse table-fixed w-full" },
                React__namespace.createElement("colgroup", null,
                    React__namespace.createElement("col", { className: "diff-table-old-num-col", width: Math.round(width) + 25 }),
                    React__namespace.createElement("col", { className: "diff-table-old-content-col" }),
                    React__namespace.createElement("col", { className: "diff-table-new-num-col", width: Math.round(width) + 25 }),
                    React__namespace.createElement("col", { className: "diff-table-new-content-col" })),
                React__namespace.createElement("thead", { className: "hidden" },
                    React__namespace.createElement("tr", null,
                        React__namespace.createElement("th", { scope: "col" }, "old line number"),
                        React__namespace.createElement("th", { scope: "col" }, "old line content"),
                        React__namespace.createElement("th", { scope: "col" }, "new line number"),
                        React__namespace.createElement("th", { scope: "col" }, "new line content"))),
                React__namespace.createElement("tbody", { className: "diff-table-body leading-[1.4]", onMouseDownCapture: onMouseDown },
                    lines.map((line) => (React__namespace.createElement(React.Fragment, { key: line.index },
                        React__namespace.createElement(DiffSplitHunkLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React__namespace.createElement(DiffSplitLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React__namespace.createElement(DiffSplitWidgetLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React__namespace.createElement(DiffSplitExtendLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile })))),
                    React__namespace.createElement(DiffSplitLastHunkLine, { diffFile: diffFile }))))));
});
DiffSplitViewWrap.displayName = "DiffSplitViewWrap";

const DiffSplitView = React.memo(({ diffFile }) => {
    const { useDiffContext } = useDiffViewContext();
    const enableWrap = useDiffContext(React__namespace.useCallback((s) => s.enableWrap, []));
    // performance optimization
    const useWidget = React.useMemo(() => reactivityStore.createStore(() => {
        const widgetSide = reactivityStore.ref(undefined);
        const widgetLineNumber = reactivityStore.ref(undefined);
        const setWidget = ({ side, lineNumber }) => {
            widgetSide.value = side;
            widgetLineNumber.value = lineNumber;
        };
        return { widgetSide, widgetLineNumber, setWidget };
    }), []);
    const contextValue = React.useMemo(() => ({ useWidget }), [useWidget]);
    shim.useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    React.useEffect(() => {
        const { setWidget } = useWidget.getReadonlyState();
        setWidget({});
    }, [diffFile, useWidget]);
    return (React__namespace.createElement(DiffWidgetContext.Provider, { value: contextValue }, enableWrap ? React__namespace.createElement(DiffSplitViewWrap, { diffFile: diffFile }) : React__namespace.createElement(DiffSplitViewNormal, { diffFile: diffFile })));
});
DiffSplitView.displayName = "DiffSplitView";

const _DiffUnifiedExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const renderExtendLine = useDiffContext(React.useCallback((s) => s.renderExtendLine, []));
    const unifiedItem = diffFile.getUnifiedLine(index);
    const oldExtend = useDiffContext(React.useCallback((s) => s.extendData?.oldFile?.[unifiedItem?.oldLineNumber], [unifiedItem?.oldLineNumber]));
    const newExtend = useDiffContext(React.useCallback((s) => s.extendData?.newFile?.[unifiedItem?.newLineNumber], [unifiedItem?.newLineNumber]));
    const width = useDomWidth({
        selector: ".unified-diff-table-wrapper",
        enable: typeof renderExtendLine === "function",
    });
    if (!renderExtendLine)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", className: "diff-line diff-line-extend" },
        React__namespace.createElement("td", { className: "diff-line-extend-content align-top p-0", colSpan: 2 },
            React__namespace.createElement("div", { className: "diff-line-extend-wrapper sticky left-0", style: { width } },
                width > 0 &&
                    oldExtend &&
                    renderExtendLine?.({
                        diffFile,
                        side: exports.SplitSide.old,
                        lineNumber: unifiedItem.oldLineNumber,
                        data: oldExtend.data,
                        onUpdate: diffFile.notifyAll,
                    }),
                width > 0 &&
                    newExtend &&
                    renderExtendLine?.({
                        diffFile,
                        side: exports.SplitSide.new,
                        lineNumber: unifiedItem.newLineNumber,
                        data: newExtend.data,
                        onUpdate: diffFile.notifyAll,
                    })))));
};
const DiffUnifiedExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const unifiedItem = diffFile.getUnifiedLine(index);
    const hasExtend = useDiffContext(React.useCallback((s) => s.extendData?.oldFile?.[unifiedItem?.oldLineNumber] || s.extendData?.newFile?.[unifiedItem?.newLineNumber], [unifiedItem.oldLineNumber, unifiedItem.newLineNumber]));
    if (!hasExtend || !unifiedItem || unifiedItem.isHidden || !unifiedItem.diff)
        return null;
    return React__namespace.createElement(_DiffUnifiedExtendLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffUnifiedHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getUnifiedHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    const { useDiffContext } = useDiffViewContext();
    const enableWrap = useDiffContext(React.useCallback((s) => s.enableWrap, []));
    const isExpandAll = currentHunk &&
        currentHunk.unifiedInfo &&
        currentHunk.unifiedInfo.endHiddenIndex - currentHunk.unifiedInfo.startHiddenIndex < composeLen;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React__namespace.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[100px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, expandEnabled ? (isFirstLine ? (React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onUnifiedHunkExpand("up", index) },
            React__namespace.createElement(ExpandUp, { className: "fill-current" }))) : isExpandAll ? (React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onUnifiedHunkExpand("all", index) },
            React__namespace.createElement(ExpandAll, { className: "fill-current" }))) : (React__namespace.createElement(React__namespace.Fragment, null,
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onUnifiedHunkExpand("down", index) },
                React__namespace.createElement(ExpandDown, { className: "fill-current" })),
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onUnifiedHunkExpand("up", index) },
                React__namespace.createElement(ExpandUp, { className: "fill-current" }))))) : null),
        React__namespace.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React__namespace.createElement("div", { className: "pl-[1.5em]", style: {
                    whiteSpace: enableWrap ? "pre-wrap" : "pre",
                    wordBreak: enableWrap ? "break-all" : "initial",
                    color: `var(${hunkContentColorName})`,
                } }, currentHunk.unifiedInfo.plainText))));
};
const DiffUnifiedHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getUnifiedHunkLine(index);
    const currentIsShow = currentHunk &&
        currentHunk.unifiedInfo &&
        currentHunk.unifiedInfo.startHiddenIndex < currentHunk.unifiedInfo.endHiddenIndex;
    if (!currentIsShow)
        return null;
    return React__namespace.createElement(_DiffUnifiedHunkLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};
const DiffUnifiedLastHunkLine = ({ diffFile }) => {
    const currentIsShow = diffFile.getNeedShowExpandAll("unified");
    const expandEnabled = diffFile.getExpandEnabled();
    if (!currentIsShow || !expandEnabled)
        return null;
    return (React__namespace.createElement("tr", { "data-line": "last-hunk", "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React__namespace.createElement("td", { className: "diff-line-hunk-action sticky left-0 w-[1%] min-w-[100px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } },
            React__namespace.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onUnifiedLastExpand() },
                React__namespace.createElement(ExpandDown, { className: "fill-current" }))),
        React__namespace.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React__namespace.createElement("span", null, "\u2002"))));
};

const DiffUnifiedOldLine = ({ index, diffLine, rawLine, syntaxLine, lineNumber, diffFile, setWidget, enableWrap, enableAddWidget, enableHighlight, onAddWidgetClick, }) => {
    return (React__namespace.createElement("tr", { "data-line": index, "data-state": "diff", className: "diff-line group" },
        React__namespace.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                color: `var(${plainLineNumberColorName})`,
                backgroundColor: `var(${delLineNumberBGName})`,
            } },
            enableAddWidget && (React__namespace.createElement(DiffUnifiedAddWidget, { index: index - 1, lineNumber: lineNumber, diffFile: diffFile, side: exports.SplitSide.old, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
            React__namespace.createElement("div", { className: "flex" },
                React__namespace.createElement("span", { "data-line-old-num": lineNumber, className: "inline-block w-[50%]" }, lineNumber),
                React__namespace.createElement("span", { className: "w-[10px] shrink-0" }),
                React__namespace.createElement("span", { className: "inline-block w-[50%]" }))),
        React__namespace.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: { backgroundColor: `var(${delContentBGName})` } },
            React__namespace.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
};
const DiffUnifiedNewLine = ({ index, diffLine, rawLine, syntaxLine, lineNumber, diffFile, setWidget, enableWrap, enableAddWidget, enableHighlight, onAddWidgetClick, }) => {
    return (React__namespace.createElement("tr", { "data-line": index, "data-state": "diff", className: "diff-line group" },
        React__namespace.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                color: `var(${plainLineNumberColorName})`,
                backgroundColor: `var(${addLineNumberBGName})`,
            } },
            enableAddWidget && (React__namespace.createElement(DiffUnifiedAddWidget, { index: index - 1, lineNumber: lineNumber, diffFile: diffFile, side: exports.SplitSide.new, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
            React__namespace.createElement("div", { className: "flex" },
                React__namespace.createElement("span", { className: "inline-block w-[50%]" }),
                React__namespace.createElement("span", { className: "shrink-0 w-[10px]" }),
                React__namespace.createElement("span", { "data-line-new-num": lineNumber, className: "inline-block w-[50%]" }, lineNumber))),
        React__namespace.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: { backgroundColor: `var(${addContentBGName})` } },
            React__namespace.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
};
const _DiffUnifiedLine = React.memo(({ index, diffFile, lineNumber }) => {
    const unifiedLine = diffFile.getUnifiedLine(index);
    const { useDiffContext } = useDiffViewContext();
    const { enableWrap, enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React__namespace.useCallback((s) => ({
        enableWrap: s.enableWrap,
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const hasDiff = unifiedLine.diff;
    const hasChange = checkDiffLineIncludeChange(unifiedLine.diff);
    const rawLine = unifiedLine.value || "";
    const diffLine = unifiedLine.diff;
    const newLineNumber = unifiedLine.newLineNumber;
    const oldLinenumber = unifiedLine.oldLineNumber;
    const syntaxLine = newLineNumber
        ? diffFile.getNewSyntaxLine(newLineNumber)
        : oldLinenumber
            ? diffFile.getOldSyntaxLine(oldLinenumber)
            : undefined;
    if (hasChange) {
        if (unifiedLine.oldLineNumber) {
            return (React__namespace.createElement(DiffUnifiedOldLine, { index: lineNumber, enableWrap: enableWrap, diffFile: diffFile, rawLine: rawLine, diffLine: diffLine, setWidget: setWidget, syntaxLine: syntaxLine, enableHighlight: enableHighlight, enableAddWidget: enableAddWidget, onAddWidgetClick: onAddWidgetClick, lineNumber: unifiedLine.oldLineNumber }));
        }
        else {
            return (React__namespace.createElement(DiffUnifiedNewLine, { index: lineNumber, enableWrap: enableWrap, rawLine: rawLine, diffLine: diffLine, diffFile: diffFile, setWidget: setWidget, syntaxLine: syntaxLine, enableHighlight: enableHighlight, enableAddWidget: enableAddWidget, onAddWidgetClick: onAddWidgetClick, lineNumber: unifiedLine.newLineNumber }));
        }
    }
    else {
        return (React__namespace.createElement("tr", { "data-line": lineNumber, "data-state": unifiedLine.diff ? "diff" : "plain", className: "diff-line group" },
            React__namespace.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                    color: `var(${plainLineNumberColorName})`,
                    backgroundColor: hasDiff ? `var(${plainLineNumberBGName})` : `var(${expandContentBGName})`,
                } },
                enableAddWidget && hasDiff && (React__namespace.createElement(DiffUnifiedAddWidget, { index: index, diffFile: diffFile, lineNumber: unifiedLine.newLineNumber, side: exports.SplitSide.new, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
                React__namespace.createElement("div", { className: "flex opacity-[0.5]" },
                    React__namespace.createElement("span", { "data-line-old-num": unifiedLine.oldLineNumber, className: "inline-block w-[50%]" }, unifiedLine.oldLineNumber),
                    React__namespace.createElement("span", { className: "w-[10px] shrink-0" }),
                    React__namespace.createElement("span", { "data-line-new-num": unifiedLine.newLineNumber, className: "inline-block w-[50%]" }, unifiedLine.newLineNumber))),
            React__namespace.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: {
                    backgroundColor: hasDiff ? `var(${plainContentBGName})` : `var(${expandContentBGName})`,
                } },
                React__namespace.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
    }
});
_DiffUnifiedLine.displayName = "_DiffUnifiedLine";
const DiffUnifiedLine = ({ index, diffFile, lineNumber, }) => {
    const unifiedLine = diffFile.getUnifiedLine(index);
    if (unifiedLine?.isHidden)
        return null;
    return React__namespace.createElement(_DiffUnifiedLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffUnifiedWidgetLine = ({ index, diffFile, oldWidget, newWidget, lineNumber, setWidget, }) => {
    const unifiedItem = diffFile.getUnifiedLine(index);
    const onClose = () => setWidget({});
    const { useDiffContext } = useDiffViewContext();
    // 需要显示的时候才进行方法订阅，可以大幅度提高性能
    const renderWidgetLine = useDiffContext(React.useCallback((s) => s.renderWidgetLine, []));
    const width = useDomWidth({
        selector: ".unified-diff-table-wrapper",
        enable: typeof renderWidgetLine === "function",
    });
    if (!renderWidgetLine)
        return null;
    return (React__namespace.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", className: "diff-line diff-line-widget" },
        React__namespace.createElement("td", { className: "diff-line-widget-content p-0", colSpan: 2 },
            React__namespace.createElement("div", { className: "diff-line-widget-wrapper sticky left-0", style: { width } },
                width > 0 &&
                    oldWidget &&
                    renderWidgetLine?.({ diffFile, side: exports.SplitSide.old, lineNumber: unifiedItem.oldLineNumber, onClose }),
                width > 0 &&
                    newWidget &&
                    renderWidgetLine?.({ diffFile, side: exports.SplitSide.new, lineNumber: unifiedItem.newLineNumber, onClose })))));
};
const DiffUnifiedWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetSide, widgetLineNumber, setWidget } = useWidget(React__namespace.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const unifiedItem = diffFile.getUnifiedLine(index);
    const oldWidget = unifiedItem.oldLineNumber && widgetSide === exports.SplitSide.old && widgetLineNumber === unifiedItem.oldLineNumber;
    const newWidget = unifiedItem.newLineNumber && widgetSide === exports.SplitSide.new && widgetLineNumber === unifiedItem.newLineNumber;
    const currentIsShow = oldWidget || newWidget;
    if (!currentIsShow)
        return null;
    return (React__namespace.createElement(_DiffUnifiedWidgetLine, { index: index, diffFile: diffFile, lineNumber: lineNumber, oldWidget: oldWidget, newWidget: newWidget, setWidget: setWidget }));
};

const onMouseDown = (e) => {
    const ele = e.target;
    // need remove all the selection
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
        removeAllSelection();
        return;
    }
};
const DiffUnifiedView = React.memo(({ diffFile }) => {
    // performance optimization
    const useWidget = React.useMemo(() => reactivityStore.createStore(() => {
        const widgetSide = reactivityStore.ref(undefined);
        const widgetLineNumber = reactivityStore.ref(undefined);
        const setWidget = ({ side, lineNumber }) => {
            widgetSide.value = side;
            widgetLineNumber.value = lineNumber;
        };
        return { widgetSide, widgetLineNumber, setWidget };
    }), []);
    const contextValue = React.useMemo(() => ({ useWidget }), [useWidget]);
    shim.useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    React.useEffect(() => {
        const { setWidget } = useWidget.getReadonlyState();
        setWidget({});
    }, [diffFile, useWidget]);
    const lines = getUnifiedContentLine(diffFile);
    return (React__namespace.createElement(DiffWidgetContext.Provider, { value: contextValue },
        React__namespace.createElement("div", { className: "unified-diff-view w-full" },
            React__namespace.createElement("div", { className: "unified-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", style: { fontFamily: "Menlo, Consolas, monospace", fontSize: "var(--diff-font-size--)" } },
                React__namespace.createElement("table", { className: "unified-diff-table border-collapse w-full" },
                    React__namespace.createElement("colgroup", null,
                        React__namespace.createElement("col", { className: "unified-diff-table-num-col" }),
                        React__namespace.createElement("col", { className: "unified-diff-table-content-col" })),
                    React__namespace.createElement("thead", { className: "hidden" },
                        React__namespace.createElement("tr", null,
                            React__namespace.createElement("th", { scope: "col" }, "line number"),
                            React__namespace.createElement("th", { scope: "col" }, "line content"))),
                    React__namespace.createElement("tbody", { className: "diff-table-body  leading-[1.4]", onMouseDownCapture: onMouseDown },
                        lines.map((item) => (React__namespace.createElement(React.Fragment, { key: item.index },
                            React__namespace.createElement(DiffUnifiedHunkLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React__namespace.createElement(DiffUnifiedLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React__namespace.createElement(DiffUnifiedWidgetLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React__namespace.createElement(DiffUnifiedExtendLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile })))),
                        React__namespace.createElement(DiffUnifiedLastHunkLine, { diffFile: diffFile })))))));
});
DiffUnifiedView.displayName = "DiffUnifiedView";

/* eslint-disable @typescript-eslint/no-unnecessary-type-constraint */
/* eslint-disable @typescript-eslint/ban-ts-comment */
const diffFontSizeName = "--diff-font-size--";
exports.SplitSide = void 0;
(function (SplitSide) {
    SplitSide[SplitSide["old"] = 1] = "old";
    SplitSide[SplitSide["new"] = 2] = "new";
})(exports.SplitSide || (exports.SplitSide = {}));
const _InternalDiffView = (props) => {
    const { diffFile, className, style, diffViewMode, diffViewWrap, diffViewFontSize, diffViewHighlight, renderWidgetLine, renderExtendLine, extendData, diffViewAddWidget, onAddWidgetClick, } = props;
    const diffFileId = React.useMemo(() => diffFile.getId(), [diffFile]);
    // performance optimization
    const useDiffContext = React.useMemo(() => reactivityStore.createStore(() => {
        const id = reactivityStore.ref(diffFileId);
        const setId = (_id) => (id.value = _id);
        const mode = reactivityStore.ref(props.diffViewMode);
        const setMode = (_mode) => (mode.value = _mode);
        const enableWrap = reactivityStore.ref(props.diffViewWrap);
        const setEnableWrap = (_enableWrap) => (enableWrap.value = _enableWrap);
        const enableAddWidget = reactivityStore.ref(props.diffViewAddWidget);
        const setEnableAddWidget = (_enableAddWidget) => (enableAddWidget.value = _enableAddWidget);
        const enableHighlight = reactivityStore.ref(props.diffViewHighlight);
        const setEnableHighlight = (_enableHighlight) => (enableHighlight.value = _enableHighlight);
        const fontSize = reactivityStore.ref(props.diffViewFontSize);
        const setFontSize = (_fontSize) => (fontSize.value = _fontSize);
        const extendData = reactivityStore.ref({
            oldFile: { ...props.extendData?.oldFile },
            newFile: { ...props.extendData?.newFile },
        });
        const setExtendData = (_extendData) => {
            const existOldKeys = Object.keys(extendData.value.oldFile || {});
            const inComingOldKeys = Object.keys(_extendData.oldFile || {});
            for (const key of existOldKeys) {
                if (!inComingOldKeys.includes(key)) {
                    delete extendData.value.oldFile[key];
                }
            }
            for (const key of inComingOldKeys) {
                extendData.value.oldFile[key] = _extendData.oldFile[key];
            }
            const existNewKeys = Object.keys(extendData.value.newFile || {});
            const inComingNewKeys = Object.keys(_extendData.newFile || {});
            for (const key of existNewKeys) {
                if (!inComingNewKeys.includes(key)) {
                    delete extendData.value.newFile[key];
                }
            }
            for (const key of inComingNewKeys) {
                extendData.value.newFile[key] = _extendData.newFile[key];
            }
        };
        const renderWidgetLine = reactivityStore.ref(props.renderWidgetLine);
        const setRenderWidgetLine = (_renderWidgetLine) => (renderWidgetLine.value = _renderWidgetLine);
        const renderExtendLine = reactivityStore.ref(props.renderExtendLine);
        const setRenderExtendLine = (_renderExtendLine) => (renderExtendLine.value = _renderExtendLine);
        const onAddWidgetClick = reactivityStore.ref(props.onAddWidgetClick);
        const setOnAddWidgetClick = (_onAddWidgetClick) => (onAddWidgetClick.value = _onAddWidgetClick);
        return {
            id,
            setId,
            mode,
            setMode,
            enableWrap,
            setEnableWrap,
            enableAddWidget,
            setEnableAddWidget,
            enableHighlight,
            setEnableHighlight,
            fontSize,
            setFontSize,
            extendData,
            setExtendData,
            renderWidgetLine,
            setRenderWidgetLine,
            renderExtendLine,
            setRenderExtendLine,
            onAddWidgetClick,
            setOnAddWidgetClick,
        };
    }), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    React.useEffect(() => {
        const { id, setId, mode, setMode, enableAddWidget, setEnableAddWidget, enableHighlight, setEnableHighlight, enableWrap, setEnableWrap, setExtendData, fontSize, setFontSize, onAddWidgetClick: _onAddWidgetClick, setOnAddWidgetClick, renderExtendLine: _renderExtendLine, setRenderExtendLine, renderWidgetLine: _renderWidgetLine, setRenderWidgetLine, } = useDiffContext.getReadonlyState();
        diffFileId && diffFileId !== id && setId(diffFileId);
        diffViewMode && diffViewMode !== mode && setMode(diffViewMode);
        diffViewAddWidget !== enableAddWidget && setEnableAddWidget(diffViewAddWidget);
        diffViewHighlight !== enableHighlight && setEnableHighlight(diffViewHighlight);
        diffViewWrap !== enableWrap && setEnableWrap(diffViewWrap);
        extendData && setExtendData(extendData);
        diffViewFontSize && diffViewFontSize !== fontSize && setFontSize(diffViewFontSize);
        onAddWidgetClick !== _onAddWidgetClick && setOnAddWidgetClick(onAddWidgetClick);
        renderExtendLine !== _renderExtendLine && setRenderExtendLine(renderExtendLine);
        renderWidgetLine !== _renderWidgetLine && setRenderWidgetLine(renderWidgetLine);
    }, [
        useDiffContext,
        diffViewFontSize,
        diffViewHighlight,
        diffViewMode,
        diffViewWrap,
        diffViewAddWidget,
        diffFileId,
        renderWidgetLine,
        renderExtendLine,
        extendData,
        onAddWidgetClick,
    ]);
    useUnmount(() => diffFile.clearId(), [diffFile]);
    const value = React.useMemo(() => ({ useDiffContext }), [useDiffContext]);
    return (React__namespace.createElement(DiffViewContext.Provider, { value: value },
        React__namespace.createElement("div", { className: "diff-tailwindcss-wrapper" },
            React__namespace.createElement("div", { className: "diff-style-root", style: {
                    // @ts-ignore
                    [diffFontSizeName]: diffViewFontSize + "px",
                } },
                React__namespace.createElement("div", { id: `diff-root${diffFileId}`, className: "diff-view-wrapper" + (className ? ` ${className}` : ""), style: style }, diffViewMode === exports.DiffModeEnum.Split ? (React__namespace.createElement(DiffSplitView, { diffFile: diffFile })) : (React__namespace.createElement(DiffUnifiedView, { diffFile: diffFile })))))));
};
const InternalDiffView = React.memo(_InternalDiffView);
const DiffView = (props) => {
    const { registerHighlighter, autoDetectLang, data, diffFile: _diffFile, ...restProps } = props;
    const diffFile = React.useMemo(() => {
        if (_diffFile) {
            const diffFile = DiffFile.createInstance({});
            diffFile._mergeFullBundle(_diffFile._getFullBundle());
            return diffFile;
        }
        else if (data) {
            return new DiffFile(data?.oldFile?.fileName || "", data?.oldFile?.content || "", data?.newFile?.fileName || "", data?.newFile?.content || "", data?.hunks || [], data?.oldFile?.fileLang || "", data?.newFile?.fileLang || "");
        }
        return null;
    }, [data, _diffFile]);
    React.useEffect(() => {
        if (!diffFile)
            return;
        diffFile.initRaw();
        diffFile.buildSplitDiffLines();
        diffFile.buildUnifiedDiffLines();
    }, [diffFile]);
    React.useEffect(() => {
        if (!diffFile)
            return;
        if (props.diffViewHighlight) {
            diffFile.initSyntax({ autoDetectLang, registerHighlighter });
            diffFile.notifyAll();
        }
    }, [diffFile, props.diffViewHighlight, autoDetectLang, registerHighlighter]);
    if (!diffFile)
        return null;
    return (React__namespace.createElement(InternalDiffView, { key: diffFile.getId(), ...restProps, diffFile: diffFile, diffViewFontSize: restProps.diffViewFontSize || 14 }));
};
const version = "0.0.2";

exports.DiffView = DiffView;
exports.DiffViewContext = DiffViewContext;
exports.useDiffViewContext = useDiffViewContext;
exports.version = version;
//# sourceMappingURL=index.development.js.map
